#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""PyFRAP module for simple geometric operations, such as

	* Angle computation.
	* Normal computation.
	
"""
#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#Numpy/Scipy
import numpy as np

#String
import string

#PyFRAP modules
from pyfrp_term_module import *
import pyfrp_misc_module

#===========================================================================================================================================================================
#Module Functions
#===========================================================================================================================================================================

def getAngle(vec1,vec2):
	
	r"""Returns angle between two vectors in radians.
	
	Angle is calculated via
	
	.. math:: \phi = \frac{v_1 \dot v_2}{|v_1| |v_2|}
	
	.. note:: Checks for numerical errors and corrects them if necessary.
	
	Args:
		vec1 (numpy.ndarray): Vector 1.
		vec2 (numpy.ndarray): Vector 2.
	
	Returns: 
		float: Angle.
	
	"""

	x=np.dot(vec1,vec2)/(np.linalg.norm(vec1)*np.linalg.norm(vec2))
	
	if x>1.0:
		if x>1.+1E-4:
			printError("getAngle: x out of bounds.")
		x=1.0
	
	if x==1.0:
		return 0.0
	if x==-1.0:
		return np.pi
	
	a=np.arccos(x)
	
	if a<0:
		return getAngle(vec2,vec1)

	return a

def flipCoordinate(x,destAxis,origAxis='x',debug=False):
	
	"""Transforms coodinate from one axis to another by
	rolling the coordinates, e.g. clockwise turning the 
	point.
	
	``destAxis`` and ``origAxis`` are given as one of 
	``x,y,z``. 
	
	Args:
		x (numpy.ndarray): Coordinate to turn.
		destAxis (str): Destination axis.
		
	Keyword Args:	
		origAxis (str): Original axis.
		debug (bool): Print debugging output.
	
	Returns:
		numpy.ndarray: Transformed coordinate.
	
	"""
	
		
	# Calculate differences between axis
	axisDiff=abs(string.lowercase.index(destAxis)-string.lowercase.index(origAxis))
	
	# Roll
	xnew=np.roll(x,axisDiff)
	
	# Print debugging messages
	if debug:
		print "Transforming coordinate " , x, " from axis ", origAxis, " to axis ", destAxis , "."
		print "axisDiff = ", axisDiff
		print "xnew = ", xnew
	
	return xnew 

def newellsMethod(vertices):
	
	"""Computes normal using Newell's method.
		
	Adapted from http://stackoverflow.com/questions/39001642/calculating-surface-normal-in-python-using-newells-method.
	
	Vertices should be given as 
	
	>>> vertices=[[x1,y1,z1],[x2,y2,z2],...]
	
	Args:
		vertices (list): List of vertex coordinates.
	
	Returns:
		numpy.ndarray: Normal vector to surface.
	"""
	
	#Newell's method
	n = [0.0, 0.0, 0.0]

	for i, x in enumerate(vertices):
		
		x2 = vertices[(i+1) % len(vertices)]
		
		nBefore=list(n)
		
		n[0] += (x[1] - x2[1]) * (x[2] + x2[2])
		n[1] += (x[2] - x2[2]) * (x[0] - x2[0])
		n[2] += (x[0] - x2[0]) * (x[1] - x2[1])
		
		
		#This is for closed loops that somehow return a zero normal vector.
		#Don't know if this is a good workaround
		if sum(nBefore)!=0 and sum(n)==0:
			n=list(nBefore)
		
	normalised = [i/sum(n) for i in n]
	
	return normalised

def computeNormal(vertices,method="cross"):
	
	"""Computes normal.

	Vertices should be given as 
	
	>>> vertices=[[x1,y1,z1],[x2,y2,z2],...]
	
	Currently there are two methods available:
		
		* ``cross``, see also :py:func:`normalByCross`.
		* ``newells``, see also :py:func:`newells`.
	
	If method is unknown, will fall back to ``cross``.
	
	Args:
		vertices (list): List of vertex coordinates.
	
	Keyword Args:
		method (str): Method of normal computation.
	
	Returns:
		numpy.ndarray: Normal vector.
	"""
	
	if method.lower() not in ["cross","newells"]:
		printWarning("Unknown method " + method + " for normal computation. Will use method=cross." )
		method=cross
		
	if method.lower()=="cross":
		vec1=np.array(vertices[1])-np.array(vertices[0])
		vec2=np.array(vertices[2])-np.array(vertices[0])

		return normalByCross(vec1,vec2)
		
	elif method.lower()=="newells":
		return newellsMethod(vertices)
		
def normalByCross(vec1,vec2):
	
	r"""Returns normalised normal vectors of plane spanned by two vectors.
	
	Normal vector is computed by:
	
	.. math:: \mathbf{n} = \frac{\mathbf{v_1} \times \mathbf{v_2}}{|\mathbf{v_1} \times \mathbf{v_2}|}
	
	.. note:: Will return zero vector if ``vec1`` and ``vec2`` are colinear.
	
	Args:
		vec1 (numpy.ndarray): Vector 1.
		vec2 (numpy.ndarray): Vector 2.
	
	Returns:
		numpy.ndarray: Normal vector.
	"""
	
	if checkColinear(vec1,vec2):
		
		printWarning("Can't compute normal of vectors, they seem to be colinear. Returning zero.")
		return np.zeros(np.shape(vec1))
		
	return np.cross(vec1,vec2)/np.linalg.norm(np.cross(vec1,vec2))
	
def checkColinear(vec1,vec2):
	
	"""Returns True if two vectors are colinear.
	
	Args:
		vec1 (numpy.ndarray): Vector 1.
		vec2 (numpy.ndarray): Vector 2.
	
	Returns:
		bool: True if colinear.
	
	"""
	
	return sum(np.cross(vec1,vec2))==0
 
def getRotMatrix(n1,n2):

	"""Builds rotation matrix for the rotation of a vector n2 
	onto n1.
	
	Taken from http://stackoverflow.com/questions/9423621/3d-rotations-of-a-plane .
	
	Args:
		n1 (numpy.ndarray): Vector to be rotated to.
		n2 (numpy.ndarray): Vector to rotate.
		
	Returns:
		numpy.ndarray: Rotation matrix

	"""
	
	# Rotation axis
	rotAxis=np.cross(n2,n1)
	rotAxis=rotAxis/np.linalg.norm(rotAxis)
	
	#Rotation angle
	c=np.dot(n2,n1)/(np.linalg.norm(n2)*np.linalg.norm(n1))
	s=np.sqrt(1-c**2)
	C=1-c
	
	x=rotAxis[0]
	y=rotAxis[1]
	z=rotAxis[2]
	
	# Build rotation matrix
	rmat = np.array([[x*x*C+c,x*y*C-z*s,x*z*C+y*s],[y*x*C+z*s,y*y*C+c,y*z*C-x*s],[z*x*C-y*s,z*y*C+x*s,z*z*C+c]])
	
	return rmat.T

def decodeEuclideanBase(d):
	
	"""Decodes a euclidean base vector given as a literal.
	
	Example:
	
	>>> decodeEuclideanBase('z')
	>>> array([ 0.,  0.,  1.])
	
	Args:
		d (str): Direction ("x"/"y"/"z")
		
	Returns:
		numpy.ndarray: Base vector.
	
	"""
	
	if d=='x':
		return np.array([1.,0.,0.])
	elif d=='y':
		return np.array([0.,1.,0.])
	elif d=='z':
		return np.array([0.,0.,1.])
	else:
		printError("Unknown direction "+d)
		raise ValueError
		


	


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#PyQT Dialogs for editing embryo datasets
#(1) embryoDialog

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#QT
from PyQt4 import QtGui, QtCore

#PyFRAP GUI classes
import pyfrp_gui_basics

#PyFRAP modules
from pyfrp.modules.pyfrp_term_module import *
from pyfrp.modules import pyfrp_img_module
from pyfrp.modules import pyfrp_misc_module

#Numpy/Scipy
import numpy as np

#Misc 
import os

#===================================================================================================================================
#Dialog for editing embryo datasets
#===================================================================================================================================

class embryoDialog(pyfrp_gui_basics.basicCanvasDialog):
	
	def __init__(self,embryo,parent):
		
		super(embryoDialog,self).__init__(parent)
		
		#Passing embryo
		self.embryo=embryo
		self.parent=parent
		self.nCharDisplayed=50
		
		#Labels
		self.lblName = QtGui.QLabel("Name:", self)
		
		self.lblDataFT = QtGui.QLabel("Data Filetype:", self)
		self.lblDataEnc = QtGui.QLabel("Data Encoding:", self)
		
		self.lblFnDataFolder = QtGui.QLabel("Data Folder:", self)
	
		
		self.lblDataResPx = QtGui.QLabel("Resolution (px):", self)
		self.lblDataResMu = QtGui.QLabel("Resulution (um):", self)
		
		self.lblSliceDepth = QtGui.QLabel("Imaging Depth (um):", self)
		
		self.lblFrameInterval = QtGui.QLabel("Frame Interval (s):", self)
		self.lblnFrames = QtGui.QLabel("number of frames:", self)
		self.lbltStart = QtGui.QLabel("tStart (s):", self)
		self.lbltEnd = QtGui.QLabel("tEnd (s):", self)
		
		self.lblFnDataFolderValue = QtGui.QLabel("", self)
		
		self.updateDataFolderLbl()
		
		#LineEdits
		self.qleName = QtGui.QLineEdit(self.embryo.name)
		self.qleDataResPx = QtGui.QLineEdit(str(self.embryo.dataResPx))
		self.qleDataResMu = QtGui.QLineEdit(str(self.embryo.dataResMu))
		
		self.qleSliceDepth = QtGui.QLineEdit(str(self.embryo.sliceDepthMu))
		
		self.qleFrameInterval = QtGui.QLineEdit(str(self.embryo.frameInterval))
		self.qlenFrames = QtGui.QLineEdit(str(self.embryo.nFrames))
		self.qletStart = QtGui.QLineEdit(str(self.embryo.tStart))
		self.qletEnd = QtGui.QLineEdit(str(self.embryo.tEnd))
		
		self.doubleValid=QtGui.QDoubleValidator()
		
		self.qleDataResPx.setValidator(self.doubleValid)
		self.qleDataResMu.setValidator(self.doubleValid)
		self.qleSliceDepth.setValidator(self.doubleValid)
		self.qleFrameInterval.setValidator(self.doubleValid)
		self.qlenFrames.setValidator(self.doubleValid)
		self.qletStart.setValidator(self.doubleValid)
		self.qletEnd.setValidator(self.doubleValid)
		
		self.qleName.editingFinished.connect(self.setName)
		self.qleDataResMu.editingFinished.connect(self.setDataResMu)
		self.qleDataResPx.editingFinished.connect(self.setDataResPx)
		self.qleSliceDepth.editingFinished.connect(self.setSliceDepth)
		self.qleFrameInterval.editingFinished.connect(self.setFrameInterval)
		self.qletStart.editingFinished.connect(self.settStart)
		
		self.qletEnd.setReadOnly(True)
		self.qlenFrames.setReadOnly(True)
		
		#Combo
		self.comboDataFt = QtGui.QComboBox(self)
		self.comboDataFt.addItem("tif")
		self.comboDataFt.setCurrentIndex(self.comboDataFt.findText(self.embryo.getDataFT()))
		
		self.comboDataEnc = QtGui.QComboBox(self)
		self.comboDataEnc.addItem("uint8")
		self.comboDataEnc.addItem("uint16")
		self.comboDataEnc.setCurrentIndex(self.comboDataEnc.findText(self.embryo.getDataEnc())) 
		
		self.comboDataFt.activated[str].connect(self.setDataFt)   
		self.comboDataEnc.activated[str].connect(self.setDataEnc)   
	
		#Buttons
		self.btnFnDatafolder=QtGui.QPushButton('Change')
		
		self.btnFnDatafolder.connect(self.btnFnDatafolder, QtCore.SIGNAL('clicked()'), self.setFnDatafolder)
			
		#Layout
		self.dataFolderGrid = QtGui.QGridLayout()
		self.dataFolderGrid.addWidget(self.lblFnDataFolderValue,1,1)
		self.dataFolderGrid.addWidget(self.btnFnDatafolder,1,2)
		
		nRows=self.grid.rowCount()
		
		self.grid.addWidget(self.lblName,nRows+1,1)
		self.grid.addWidget(self.lblDataFT,nRows+3,1)
		self.grid.addWidget(self.lblDataEnc,nRows+4,1)
		self.grid.addWidget(self.lblFnDataFolder,nRows+5,1)
		
		self.grid.addWidget(self.lblDataResPx,nRows+8,1)
		self.grid.addWidget(self.lblDataResMu,nRows+9,1)
		
		self.grid.addWidget(self.lblSliceDepth,nRows+10,1)
		
		self.grid.addWidget(self.lblFrameInterval,nRows+12,1)
		self.grid.addWidget(self.lblnFrames,nRows+13,1)
		self.grid.addWidget(self.lbltStart,nRows+14,1)
		self.grid.addWidget(self.lbltEnd,nRows+15,1)
		
		self.grid.addWidget(self.qleName,nRows+1,2)
		self.grid.addWidget(self.comboDataFt,nRows+3,2)
		self.grid.addWidget(self.comboDataEnc,nRows+4,2)
		self.grid.addLayout(self.dataFolderGrid,nRows+5,2)
		
		self.grid.addWidget(self.qleDataResPx,nRows+8,2)
		self.grid.addWidget(self.qleDataResMu,nRows+9,2)
		
		self.grid.addWidget(self.qleSliceDepth,nRows+10,2)
		
		self.grid.addWidget(self.qleFrameInterval,nRows+12,2)
		self.grid.addWidget(self.qlenFrames,nRows+13,2)
		self.grid.addWidget(self.qletStart,nRows+14,2)
		self.grid.addWidget(self.qletEnd,nRows+15,2)
		
		self.showFirstDataImg()
		
		self.show()
		
	def setFnDatafolder(self):
		
		folder = str(QtGui.QFileDialog.getExistingDirectory(self, "Select Data Directory",  self.parent.lastopen,))
		if folder=='':
			return
		
		self.embryo.fnDatafolder=folder
		
		self.parent.lastopen=folder
		
		self.embryo.fnDatafolder=pyfrp_misc_module.slashToFn(self.embryo.fnDatafolder)
	
		self.updateEmbryo()
		self.updateTimeQles()
		
		self.updateDataFolderLbl()
		
		self.showFirstDataImg()
		
	def setFnPreImage(self):
		
		fn = str(QtGui.QFileDialog.getOpenFileName(self, 'Open file', self.parent.lastopen,))
		if fn=='':
			return
		
		self.parent.lastopen,temp=os.path.split(fn)
		
		self.embryo.fnPreimage=fn
		self.updatePreImageLbl()
		
	def updateDataFolderLbl(self):
		self.lblFnDataFolderValue.setText("..."+self.embryo.fnDatafolder[-self.nCharDisplayed:])
	
	def updatePreImageLbl(self):
		self.lblFnPreimageValue.setText("..."+self.embryo.fnPreimage[-self.nCharDisplayed:])
	
	def updateTimeQles(self):
		self.qlenFrames.setText(str(self.embryo.getNFrames()))
		self.qletStart.setText(str(self.embryo.getTStart()))
		self.qletEnd.setText(str(self.embryo.getTEnd()))
			
	def setDataFt(self,text):
		
		self.embryo.setDataFT(str(text))
		
		self.updateEmbryo()
		self.updateTimeQles()
		
		self.showFirstDataImg()
		
	def setDataEnc(self,text):
	
		self.embryo.setDataEnc("uint16")
		self.showFirstDataImg()
		
	def setName(self):
		self.embryo.setName(str(self.qleName.text()))
		return self.embryo.getName()

	def setFrameInterval(self):
		self.embryo.setFrameInterval(float(str(self.qleFrameInterval.text())))
		self.updateEmbryo()
		self.updateTimeQles()
		return self.embryo.getTvecData()
		
	def settStart(self):
		self.embryo.setTStart(float(str(self.qletStart.text())))
		self.updateEmbryo()
		self.updateTimeQles()
		return self.embryo.getTStart()
		
	def setDataResPx(self):
		self.embryo.setDataResPx(float(str(self.qleDataResPx.text())))
		
		self.ax.set_xlim([0, self.embryo.dataResPx])
		self.ax.set_ylim([0, self.embryo.dataResPx])
		self.canvas.draw()
		return self.embryo.getDataResMu()
	
	def setSliceDepth(self):
		self.embryo.setSliceDepthMu(float(str(self.qleSliceDepth.text())))
	
	def setDataResMu(self):
		self.embryo.setDataResMu(float(str(self.qleDataResMu.text())))
		return self.embryo.getDataResPx()
		
	def updateEmbryo(self):
		self.embryo.updateFileList()
		self.embryo.updateNFrames()
		self.embryo.updateTimeDimensions()
		
	def showFirstDataImg(self):
		
		self.embryo.updateFileList()
		if len(self.embryo.fileList)>0:
		
			fnImg=self.embryo.fnDatafolder+self.embryo.fileList[0]
			img=pyfrp_img_module.loadImg(fnImg,self.embryo.dataEnc)
			
			self.showImg(img)
	

class lsmWizard(pyfrp_gui_basics.basicSettingsDialog):
	
	"""Dialog for lsm2embryo wizard."""
	
	def __init__(self,parent):
		
		super(lsmWizard,self).__init__(parent)
		
		#Passing parent GUI
		self.parent=parent
		
		#Default options
		self.nCharDisplayed=50
		self.fnDatafolder=""
		self.fnDestfolder=None
		self.name=""
		self.createEmbryo=True
		self.dataFT="lsm"
		self.embryo=None
		self.nChannel=1
		
		self.recoverIdent=['recover', 'post']
		self.bleachIdent=['bleach']
		self.preIdent=['pre']
		self.colorPrefix='_c00'
		
		self.cleanUp=True
		
		#Labels
		self.lblName = QtGui.QLabel("Name:", self)
		
		self.lblDataFT = QtGui.QLabel("Data Filetype:", self)
		
		self.lblFnDataFolder = QtGui.QLabel("Data Folder:", self)
	
		self.lblFnDestFolder = QtGui.QLabel("Destination Folder:", self)

		self.lblNChannel = QtGui.QLabel("nChannel:", self)
		self.lblCreate = QtGui.QLabel("Create Embryo:", self)
		
		self.lblFnDataFolderValue = QtGui.QLabel("", self)
		self.lblFnDestFolderValue = QtGui.QLabel("", self)
		
		self.lblRecoverIdent= QtGui.QLabel("Recover Identifiers", self)
		self.lblPreIdent= QtGui.QLabel("Pre Identifiers", self)
		self.lblBleachIdent= QtGui.QLabel("Bleach Identifiers", self)
		
		self.lblColorPrefix = QtGui.QLabel("Color Prefix", self)
		self.lblCleanUp = QtGui.QLabel("Clean Up Afterwards?", self)	
			
		self.updateDataFolderLbl()
		
		#LineEdits
		self.qleName = QtGui.QLineEdit("")
		self.qleRecoverIdent = QtGui.QLineEdit(str(self.recoverIdent))
		self.qlePreIdent = QtGui.QLineEdit(str(self.preIdent))
		self.qleBleachIdent = QtGui.QLineEdit(str(self.bleachIdent))
		
		self.qleColorPrefix = QtGui.QLineEdit(self.colorPrefix)
		
		self.qleName.editingFinished.connect(self.setName)
		self.qleRecoverIdent.editingFinished.connect(self.setRecoverIdent)
		self.qlePreIdent.editingFinished.connect(self.setPreIdent)
		self.qleBleachIdent.editingFinished.connect(self.setBleachIdent)
		
		self.qleColorPrefix.editingFinished.connect(self.setColorPrefix)
		
		
		#Combo
		self.comboDataFt = QtGui.QComboBox(self)
		self.comboDataFt.addItem("lsm")
		self.comboDataFt.addItem("czi")
		
		self.comboNChannel = QtGui.QComboBox(self)
		self.comboNChannel.addItem("1")
		self.comboNChannel.addItem("2")
		self.comboNChannel.addItem("3")
		self.comboNChannel.addItem("4")
		self.comboNChannel.addItem("5")
		self.comboNChannel.addItem("6")
		
		self.comboDataFt.activated[str].connect(self.setDataFt)   
		self.comboNChannel.activated[str].connect(self.setNChannel)   
		
		#Checkboxes
		self.cbCreate = QtGui.QCheckBox('', self)
		self.connect(self.cbCreate, QtCore.SIGNAL('stateChanged(int)'), self.checkCreate)
		self.cbCreate.setCheckState(2)
		
		self.cbCleanUp = QtGui.QCheckBox('', self)
		self.connect(self.cbCleanUp, QtCore.SIGNAL('stateChanged(int)'), self.checkCleanUp)
		self.cbCleanUp.setCheckState(2)
		
		#Buttons
		self.btnFnDatafolder=QtGui.QPushButton('Change')
		self.btnFnDatafolder.connect(self.btnFnDatafolder, QtCore.SIGNAL('clicked()'), self.setFnDatafolder)
		
		self.btnFnDestfolder=QtGui.QPushButton('Change')
		self.btnFnDestfolder.connect(self.btnFnDestfolder, QtCore.SIGNAL('clicked()'), self.setFnDestfolder)
		
		self.btnBuild=QtGui.QPushButton('Build Embryo')
		self.btnBuild.connect(self.btnBuild, QtCore.SIGNAL('clicked()'), self.buildEmbryo)
		
		#Layout
		self.dataFolderGrid = QtGui.QGridLayout()
		self.dataFolderGrid.addWidget(self.lblFnDataFolderValue,1,1)
		self.dataFolderGrid.addWidget(self.btnFnDatafolder,1,2)
		
		self.destFolderGrid = QtGui.QGridLayout()
		self.destFolderGrid.addWidget(self.lblFnDestFolderValue,1,1)
		self.destFolderGrid.addWidget(self.btnFnDestfolder,1,2)
		
		nRows=self.grid.rowCount()
		self.grid.addWidget(self.lblName,nRows+1,1)
		self.grid.addWidget(self.lblFnDataFolder,nRows+3,1)
		self.grid.addWidget(self.lblFnDestFolder,nRows+4,1)
		
		self.grid.addWidget(self.lblDataFT,nRows+6,1)
		self.grid.addWidget(self.lblNChannel,nRows+7,1)
		
		self.grid.addWidget(self.lblCreate,nRows+8,1)
		
		self.grid.addWidget(self.lblRecoverIdent,nRows+10,1)
		self.grid.addWidget(self.lblBleachIdent,nRows+11,1)
		self.grid.addWidget(self.lblPreIdent,nRows+12,1)
		
		self.grid.addWidget(self.lblColorPrefix,nRows+14,1)
		self.grid.addWidget(self.lblCleanUp,nRows+15,1)
		
		
		self.grid.addWidget(self.qleName,nRows+1,2)
		self.grid.addLayout(self.dataFolderGrid,nRows+3,2)
		self.grid.addLayout(self.destFolderGrid,nRows+4,2)
		
		self.grid.addWidget(self.comboDataFt,nRows+6,2)
		self.grid.addWidget(self.comboNChannel,nRows+7,2)
		
		self.grid.addWidget(self.cbCreate,nRows+8,2)
		
		self.grid.addWidget(self.qleRecoverIdent,nRows+10,2)
		self.grid.addWidget(self.qleBleachIdent,nRows+11,2)
		self.grid.addWidget(self.qlePreIdent,nRows+12,2)
	
		self.grid.addWidget(self.qleColorPrefix,nRows+14,2)
		
		self.grid.addWidget(self.cbCleanUp,nRows+15,2)
	
		
		self.grid.addWidget(self.btnBuild,nRows+16,2)
	
	
		self.setWindowTitle('From Microscope to Embryo Wizard')
		
		self.show()
	
	def fixIdent(self,ident):
		if ident[0]!="[":
			ident="["+ident
		if ident[-1]!="]":
			ident=ident+"]"
		
		return ident
	
	def setRecoverIdent(self):
		ident=str(self.qleRecoverIdent.text())
		ident=self.fixIdent(ident)
		self.recoverIdent=pyfrp_misc_module.str2list(ident,dtype="str")[0]
		
	def setBleachIdent(self):
		ident=str(self.qleBleachIdent.text())
		ident=self.fixIdent(ident)
		self.bleachIdent=pyfrp_misc_module.str2list(ident,dtype="str")[0]
		
	def setPreIdent(self):
		ident=str(self.qlePreIdent.text())
		ident=self.fixIdent(ident)
		self.preIdent=pyfrp_misc_module.str2list(ident,dtype="str")[0]
				
	def setName(self):
		self.name=str(self.qleName.text())
		
	def setColorPrefix(self):
		self.colorPrefix=str(self.qleColorPrefix.text())
		
	def buildEmbryo(self):	
		ret=pyfrp_misc_module.buildEmbryoWizard(self.fnDatafolder, self.dataFT, self.name, nChannel=self.nChannel-1, fnDest=self.fnDestfolder, createEmbryo=self.createEmbryo, recoverIdent=self.recoverIdent,
					  bleachIdent=self.bleachIdent,preIdent=self.preIdent,colorPrefix=self.colorPrefix,cleanUp=self.cleanUp)
		
		if ret==-1:
			printError("Embryo creation was not successful.")
		else:
			self.embryo=ret
			
				
	def setFnDatafolder(self):
		
		folder = str(QtGui.QFileDialog.getExistingDirectory(self, "Select Data Directory",  self.parent.lastopen,))
		if folder=='':
			return
		
		self.fnDatafolder=folder
	
		self.parent.lastopen=folder
		
		self.fnDatafolder=pyfrp_misc_module.slashToFn(self.fnDatafolder)
		
		if self.fnDestfolder==None:
			self.fnDestfolder=self.fnDatafolder
			self.updateDestFolderLbl()
			
		if self.name=="":
			files=pyfrp_misc_module.getSortedFileList(self.fnDatafolder,self.dataFT)
			self.name=files[0].replace("."+self.dataFT,"")
			self.qleName.setText(self.name)
			
		self.updateDataFolderLbl()
			
	def updateDataFolderLbl(self):
		self.lblFnDataFolderValue.setText("..."+self.fnDatafolder[-self.nCharDisplayed:])

	def setFnDestfolder(self):
		
		folder = str(QtGui.QFileDialog.getExistingDirectory(self, "Select Dest Directory",  self.parent.lastopen,))
		if folder=='':
			return
		
		self.fnDestfolder=folder
	
		self.parent.lastopen=folder
		
		self.fnDestfolder=pyfrp_misc_module.slashToFn(self.fnDestfolder)
		
		self.updateDestFolderLbl()
			
	def updateDestFolderLbl(self):
		self.lblFnDestFolderValue.setText("..."+self.fnDestfolder[-self.nCharDisplayed:])

	
	def checkCreate(self,val):
		self.createEmbryo=(bool(2*val))
		
	def checkCleanUp(self,val):
		self.cleanUp=(bool(2*val))	
		
	def setDataFt(self,text):
		
		self.dataFT=(str(text))
		
	def setNChannel(self,text):
		self.nChannel=int(str(text))
	
	def getEmbryo(self):
		return self.embryo
	
class wizardSelector(QtGui.QDialog):
	
	"""Dialog to select if either use lsmWizard or not."""
	
	def __init__(self,parent):
		
		super(wizardSelector,self).__init__(parent)
		
		#Passing parent GUI
		self.parent=parent
		
		self.useLSM=None
		
		#Buttons
		self.btnUseLSM=QtGui.QPushButton('Create Embryo from Microscope Data')
		self.btnUseLSM.connect(self.btnUseLSM, QtCore.SIGNAL('clicked()'), self.setUseLSM)
		
		self.btnUseSorted=QtGui.QPushButton('Create Embryo from already prepared Data')
		self.btnUseSorted.connect(self.btnUseSorted, QtCore.SIGNAL('clicked()'), self.setUseSorted)
			
		self.vbox = QtGui.QVBoxLayout()
		self.vbox.addWidget(self.btnUseLSM)
		self.vbox.addWidget(self.btnUseSorted)
		
		self.setLayout(self.vbox)    
			
		self.setWindowTitle('Select Wizard Steps')    
		self.show()
		
	def setUseLSM(self):
		self.useLSM=True
		self.done(1)
		
	def setUseSorted(self):
		self.useLSM=False
		self.done(1)
		
	def getUseLSM(self):
		return self.useLSM
	
	
		
		
		
	


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#PyQT Dialogs for editing PyFRAP settings
#(1) moleculeDialog

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#QT
from PyQt4 import QtGui, QtCore

#===================================================================================================================================
#Dialog for select/edit molecule
#===================================================================================================================================
	
class pathDialog(QtGui.QDialog):
	
	"""Dialog to modify path settings.
	
	Gives user the ability to easily set a path for the paths file.
	
	"""
	
	def __init__(self,identifier,path, parent):
	
		QtGui.QDialog.__init__(self, parent)

		self.parent=parent
		self.identifier=identifier
		self.path=path
		
		#Labels
		self.lblIdentifier=QtGui.QLabel("Identifier:", self)
		self.lblPath=QtGui.QLabel("Path:", self)
		self.lblPathVal=QtGui.QLabel(self.path, self)
		
		#QLEs
		self.qleIdentifier = QtGui.QLineEdit(str(self.identifier))
		self.qleIdentifier.editingFinished.connect(self.setIdentifier)
		
		#Button
		self.btnPath=QtGui.QPushButton('Change')
		self.btnPath.connect(self.btnPath, QtCore.SIGNAL('clicked()'), self.setPath)
		
		self.btnDone=QtGui.QPushButton('Done')
		self.btnDone.connect(self.btnDone, QtCore.SIGNAL('clicked()'), self.donePressed)
		
		#Layout
		self.grid = QtGui.QGridLayout()		
		self.grid.setColumnMinimumWidth(2,200) 
		
		self.grid.addWidget(self.lblIdentifier,1,1)
		self.grid.addWidget(self.lblPath,2,1)
		self.grid.addWidget(self.lblPathVal,2,2)
		self.grid.addWidget(self.qleIdentifier,1,2)
		self.grid.addWidget(self.btnPath,2,3)
		self.grid.addWidget(self.btnDone,3,3)
		
		self.setLayout(self.grid)    
			
		self.setWindowTitle('Path Dialog')   
		
		self.show()
			
	def setIdentifier(self):
		self.identifier=str(self.qleIdentifier.text())
	
	def updateLblPathVal(self,n=50):	
		self.lblPathVal.setText("..."+self.path[-n:])		
		
	def getPath(self):
		return self.identifier,self.path
	
	def setPath(self):
		
		fn = str(QtGui.QFileDialog.getOpenFileName(self, 'Open file',self.parent.lastopen,))
		if fn=='':
			return
		
		self.path=fn
		self.updateLblPathVal()
		
	def donePressed(self):
		self.done(1)
		return			
		



#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""PyFRAP module containing geometry classes. The :py:class:`geometry` class is 
a simple geometry class providing basic parameters and methods, parenting different
more specific geometries such as:

	* :py:class:`zebrafishDomeStage`: Describing a zebrafish embryo in dome stage.
	* :py:class:`cylinder`: A simple cylinder.
	* :py:class:`xenopusBall`: A simple ball geometry (looking like a xenopus embryo).
	* :py:class:`cone`: A (truncated) cone.

For most of the geometries, this module also provides quadrant reduced versions of the geometry,
reducing the geometry to the first quadrant around the center. 

.. note:: Rules for adding new geometries:
	
	   * Always subclass from  :py:class:`geometry`
	   * Always center geometry around ``geometry.center``. That includes having defining the center
	     in the .geo file by ``center_x`` and ``center_y``.
	   * Unit is pixels.
	   * Be careful with method overwrites. Use them wisely.
	   * Include the geometries into the GUI.
	   * Make them accessable by sharing them.

"""


#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#Numpy/Scipy
import numpy as np

#PyFRAP Modules
from pyfrp.modules import pyfrp_gmsh_module
from pyfrp.modules import pyfrp_gmsh_IO_module
from pyfrp.modules import pyfrp_plot_module
from pyfrp.modules import pyfrp_misc_module
from pyfrp.modules.pyfrp_term_module import *

#OS
import os,os.path
import shutil

#Solid/Opescad
import solid
import solid.utils

#===========================================================================================================================================================================
#Class definitions
#===========================================================================================================================================================================

class geometry(object):
	
	"""Basic PyFRAP geometry class. 
	
	Stores all the necessary information to describe a geometry. Comes with helpful methods
	for 
	
		* Centering (see :py:func:`centerInImg`)
		* Defining optimal ROIs (see :py:func:`optimalAllROI`)
		* Geometry file parsing (see :py:func:`readGeoFile`)
		* Geometry plotting (see :py:func:`plotGeometry`)

	Args:
		embryo (pyfrp.subclasses.pyfrp_emrbyo.embryo): Embryo class that geometry belongs to.
		typ (str): Type of geometry.
		fnGeo (str): Path to gmsh .geo file describing the geometry.
		center (numpy.ndarray): Center of geometry.
		
	
	
	"""
	
	def __init__(self,embryo,typ,fnGeo,center,dim=3):
		
		self.typ=typ
		self.embryo=embryo
		self.fnGeo=pyfrp_misc_module.fixPath(fnGeo)
		self.center=center
		self.dim=dim
		
		self.geoFileParameters={}
	
	def getDim(self):
		
		"""Returns dimension of geometry.
		
		Returns:
			int: Dimension.
		"""
		
		return self.dim
			
	def getEmbryo(self):
		
		"""Returns :py:class:`pyfrp.subclasses.pyfrp_embryo.embryo` instance 
		that geometry belongs to."""
		
		return self.embryo
	
	def getTyp(self):
		
		"""Returns type of geometry.
		
		Returns:
			str: Type of geometry.
		"""
		
		
		return self.typ
	
	def setFnGeo(self,fn):
		
		"""Sets path to .geo file.
		
		Args:
			fn (str): Path to file.
		
		Returns:
			str: Path to file.
		"""
		
		self.fnGeo=pyfrp_misc_module.fixPath(fn)
		return self.fnGeo
	
	def getFnGeo(self):
		
		"""Returns path to .geo file.
		
		Returns:
			str: Path to file.
		"""
		
		return self.fnGeo
	
	def setCenter(self,c,updateInFile=True):
		
		"""Sets geometry center.
		
		.. note:: The geometry ``center`` attribute is then also set in
		   .geo file if ``updateInFile`` is selected.
		
		Args:
			c (numpy.ndarray): New center ``[x,y]``.
			
		Keyword Args:
			updateInFile (bool): Update center in .geo file.
			
		Returns:
			numpy.ndarray: New center.
		
		"""
		
		self.center=c
		
		if updateInFile:
			if hasattr(self,'updateGeoFile'):
				self.updateGeoFile()
		return self.center
	
	def getCenter(self):
		
		"""Returns geometry center.
		
		Returns:
			numpy.ndarray: New center.
		
		"""
		
		return self.center
	
	def center2Mid(self,updateInFile=True):
		
		"""Sets geometry center to center of image.
		
		Uses ``embryo.dataResPx`` to calculate image center.
		
		.. note:: The geometry ``center`` attribute is then also set in
		   .geo file if ``updateInFile`` is selected.
			
		Keyword Args:
			updateInFile (bool): Update center in .geo file.
			
		Returns:
			numpy.ndarray: New center.
		
		"""
		
		if mod(self.embryo.dataResPx,2)==0:
			return self.setCenter([self.embryo.dataResPx/2+0.5,self.embryo.dataResPx/2+0.5],updateInFile=updateInFile)
		else:
			return self.setCenter([self.embryo.dataResPx/2,self.embryo.dataResPx/2],updateInFile=updateInFile)
	
	def centerInImg(self):
		
		"""Sets geometry center to center of image, updates .geo file and 
		if avaialable remeshes.
		
		Uses ``embryo.dataResPx`` to calculate image center.
		
		.. note:: The geometry ``center`` attribute is then also set in
		   .geo file.
				
		Returns:
			numpy.ndarray: New center.
		
		"""
		
		oldCenter=self.getCenter()
		self.centerMid()
		a=raw_input("Change center of geometry from " + oldCenter + ' to ' + self.getCenter() + ' ? [Y/N]')
		if a=='Y':
			self.updateGeoFile()
			if None!=self.embryo.simulation:
				self.embryo.simulation.mesh.genMesh()
		
		else:
			self.setCenter(oldCenter)
		
		return 	self.getCenter()
	
	def setAllROI(self,name='All',makeNew=False,updateIdxs=False):
		
		"""Tries to set the optimal *All* ROI for a specific geometry.
		
		Keyword Args:
			name (str): Name of All ROI to look for.
			makeNew (bool): Generate a new All ROI. 
			updateIdxs (bool): Update indices of new ROI.
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.ROI: New ROI instance.
		
		"""
		
		if not hasattr(self,'optimalAllROI'):
			printWarning('Cannot set AllROI. Geometry of type ' + self.typ + ' has no method for this!')
			return None
		
		if makeNew:
			rnew=self.optimalAllROI(name)
		else:	
			r=self.embryo.getROIByName(name)
			if r==None:
				printWarning('Cannot set AllROI. ROI with name ' + name + ' does not exist!')
				return None
			
			
			incl=r.findIncluded()
				
			name=r.getName()
			Id=r.getId()
			color=r.getColor()
			asMaster=r.isMaster()
			
			rnew=self.optimalAllROI(name=name,Id=Id,color=color,asMaster=asMaster)
			
			for rincl in incl:
				ind=rincl.ROIsIncluded.index(r)
				rincl.ROIsIncluded.pop(ind)
				rincl.ROIsIncluded.insert(ind,rnew)
				
				if updateIdxs:
					rincl.updateIdxs()
			
			ind=self.embryo.ROIs.index(r)
			self.embryo.removeROI(ind)
			self.embryo.ROIs.insert(ind,self.embryo.ROIs.pop(-1))
				
		return rnew
	
	def readGeoFile(self):
		
		"""Reads the .geo file and parses it into a :py:class:`pyfrp.modules.pyfrp_gmsh_geometry.domain` 
		instance. 
		
		Returns:
			:py:class:`pyfrp.modules.pyfrp_gmsh_geometry.domain`: Domain containing geometry. 
		
		"""
		
		try:
			domain,self.geoFileParameters=pyfrp_gmsh_IO_module.readGeoFile(self.fnGeo)
		except:
			printWarning("Cannot read geo file " + self.fnGeo)
			return None
		
		return domain
	
	def plotGeometry(self,ax=None,color='k',ann=False,backend='mpl',asSphere=False,linewidth=1):
		
		"""Plots geometry in 3D.
		
		Reads the .geo file and parses it into a :py:class:`pyfrp.modules.pyfrp_gmsh_geometry.domain` 
		instance. Then draws the domain.
		
		If no axes are given via ``ax``, will create new matplotlib axes.
		
		Keyword Args:
			ax (matplotlib.axes): Axes to draw in.
			color (str): Color of plot.
			ann (bool): Show annotations.
			backend (str): Backend used for drawing.
			asSphere (bool): Draw vertices as spheres.
			linewidth (float): Line width.
		
		Returns:
			matplotlib.axes: Axes used for plotting.
		
		"""
		
		domain=self.readGeoFile()
		
		if domain==None:
			return ax
		
		#if ax==None:
			#fig,axes = pyfrp_plot_module.makeSubplot([1,1],titles=["Geometry " +self.typ],proj=['3d'])
			#ax=axes[0]
		
		ax=domain.draw(ax=ax,color=color,ann=ann,backend=backend,asSphere=asSphere,linewidth=linewidth)
		
		return ax
	
	def getZExtend(self):
		
		"""Returns extend in z-direction by reading out vertices from 
		.geo file and returning maximum and minimum z-coordinates.
		
		Returns:
			tuple: Tuple containing:
				
				* zmin (float): Minimum z-coordinate.
				* zmax (float): Maximum z-coordinate.
				
		"""
		
		domain=self.readGeoFile()
		
		z=[]
		for v in domain.vertices:
			z.append(v.x[2])	
		
		return min(z), max(z) 
	
	def getXYExtend(self):
		
		"""Returns extend in x/y-direction by reading out vertices from 
		.geo file and returning maximum and minimum x/y-coordinates.
		
		Returns:
			tuple: Tuple containing:
				
				* xmin (float): Minimum x-coordinate.
				* xmax (float): Maximum x-coordinate.
				* ymin (float): Minimum y-coordinate.
				* ymax (float): Maximum y-coordinate.
				
		"""
		
		domain=self.readGeoFile()
		
		x=[]
		y=[]
		for v in domain.vertices:
			x.append(v.x[0])	
			y.append(v.x[1])	
		
		return min(x), max(x) , min(y), max(y)
	
	def getExtend(self):
		
		"""Returns extend in x/y/z-direction. 
		
		Will call :py:func:`getXYExtend` and :py:func:`getZExtend` for it.
		
		Returns:
			tuple: Tuple containing:
				
				* xmin (float): Minimum x-coordinate.
				* xmax (float): Maximum x-coordinate.
				* ymin (float): Minimum y-coordinate.
				* ymax (float): Maximum y-coordinate.
				* zmin (float): Minimum z-coordinate.
				* zmax (float): Maximum z-coordinate.
		
		"""
		
		return list(self.getXYExtend())+list(self.getZExtend())
		
	def printDetails(self):
		
		"""Prints out all details of geometry object.
		"""
		
		print "Geometry of embryo ", self.embryo.name, " Details."
		printAllObjAttr(self)
		print 
	
	def moveGeoFile(self,fn):
		
		"""Moves geometry file to different directory.
		
		.. note:: This function actually copies the file so that files in ``pyfrp/meshfiles/`` 
		   will not be removed.
		
		Will update ``geometry.fnGeo`` to the new file location.
		
		.. note:: If existent, will also copy the corresponding mesh file.
		
		Args:
			fn (str): Path of folder where geo file is supposed to go.
		
		Returns:
			str: New file location.
		
		"""
		
		shutil.copy(self.fnGeo,fn)
		
		if os.path.isfile(self.fnGeo.replace('.geo','.msh')):
			shutil.copy(self.fnGeo.replace('.geo','.msh'),fn)
			
			if hasattr(self.embryo,'simulation'):
				self.embryo.simulation.mesh.fnMesh=os.path.join(fn,os.path.basename(self.fnGeo).replace('.geo','.msh'))
				
		self.fnGeo=os.path.join(fn,os.path.basename(self.fnGeo))
	
		return self.fnGeo
	
	def getMaxGeoID(self):
		
		"""Returns maximum ID over all elements in .geo file.
		
		Sell also :py:func:`readGeoFile` and :py:func:`pyfrp.modules.pyfrp_gmsh_geometry.domain.getAllMaxID`.
		
		Returns:
			int: Maximum ID.
		
		"""
		
		domain=self.readGeoFile()
		
		return domain.getAllMaxID()
	
	def render2Openscad(self,fn=None,segments=48):
		
		"""Generates .scad file for the geometry.
		
		.. note:: If ``fn=None``, then will use the same filename and path as .geo file.
		
		Keyword Args:
			fn (str): Output filename.
			segments (int): Number of segments used for convex hull of surface.
		
		"""
		
		if fn==None:
			fn=self.fnGeo.replace(".geo",".scad")
		
		solid.scad_render_to_file(self.genAsOpenscad(), filepath=fn,file_header='$fn = %s;' % segments, include_orig_code=False)
		
	def render2Stl(self,fn=None,segments=48):
		
		"""Generates .stl file for the geometry.
		
		.. note:: If ``fn=None``, then will use the same filename and path as .geo file.
		
		Keyword Args:
			fn (str): Output filename.
			segments (int): Number of segments used for convex hull of surface.
		
		"""
		
		if fn==None:
			fn=self.fnGeo.replace(".geo",".stl")
		
		fnStl=fn
		fnScad=fnStl.replace(".stl",".scad")
		
		self.render2Openscad(fn=fnScad,segments=segments)
		pyfrp_openscad_module.runOpenscad(fnScad,fnOut=fnStl)
		
		return fnStl
	
class zebrafishDomeStage(geometry):
	
	"""Geometry describing a zebrafish embryo in dome stage.
	
	For information about zebrafish stages, see 
	http://onlinelibrary.wiley.com/doi/10.1002/aja.1002030302/abstract;jsessionid=1EAD19FE5563DAA94E3C22C5D5BEEC85.f01t03.
	
	.. image:: ../imgs/pyfrp_geometry/dome.png
	
	The zebrafish geometry is basically described by two half-balls with different radii piled on top of each other.
	The crucial geometrical parameters are:
		
		* ``outerRadius``: Radius of the outer ball.
		* ``innerRadius``: Radius of the inner ball.
		* ``centerDist``: Distance between the centers of the two balls.
	
	PyFRAP automatically computes these parameters given 
	
		* ``imagingRadius``: Radius of embryo at imaging depth.
		* ``imagingHeight``: Imaging depth.
		* ``radiusScale``: Scaling factor between radii.
		
	For details of this computations, see :py:func:`computeDome`.
		

	"""
	
	def __init__(self,embryo,center,imagingRadius,radiusScale=1.1):
		
		mdir=pyfrp_misc_module.getMeshfilesDir()
		
		super(zebrafishDomeStage, self).__init__(embryo,"zebrafishDomeStage",mdir+"dome.geo",center)
		
		#How much bigger is the inner radius than the outerRadius
		self.radiusScale=radiusScale
		self.imagingRadius=imagingRadius
	
		#Compute geometry properties
		self.restoreDefault()
		
		self.updateGeoFile()
	
	def setOuterRadius(self,r):
		
		"""Sets outer radius and updates zebrafish geometry.
		
		Args:
			r (float): New radius.
		
		Returns:
			float: New radius.
		
		"""
		
		self.outerRadius=r
		self.computeDome()
		return self.outerRadius
	
	def setRadiusScale(self,s):
		
		"""Sets scaling factor between outer and inner radius 
		and updates zebrafish geometry.
		
		Args:
			s (float): New scaling factor.
		
		Returns:
			float: New scaling factor.
		
		"""
		
		self.radiusScale=s
		self.computeDome()
		return self.radiusScale
	
	def setImagingRadius(self,r):
		
		"""Sets imaging radius and updates zebrafish geometry.
		
		Args:
			r (float): New radius.
		
		Returns:
			float: New radius.
		
		"""
		
		self.imagingRadius=r
		self.computeDome()
		return self.imagingRadius
	
	def setImagingHeight(self,h):
		
		"""Sets imaging height and updates zebrafish geometry.
		
		Args:
			h (float): New height.
		
		Returns:
			float: New height.
		
		"""
		
		self.imagingHeight=h
		if self.embryo.sliceHeightPx!=self.imagingHeight:
			printWarning("imagingHeight of geometry is not identical with imaging depth of embryo. This can lead to problems.")
		
		self.computeDome()
		return self.imagingHeight
	
	def getImagingRadius(self):
		
		"""Returns imaging radius.
		"""
		
		return self.imagingRadius
	
	def getImagingHeight(self):
		
		"""Returns imaging height.
		"""
		
		return self.imagingHeight
	
	def getOuterRadius(self):
		
		"""Returns outer radius.
		"""
		
		return self.outerRadius
	
	def getInnerRadius(self):
		
		"""Returns inner radius.
		"""
		
		return self.innerRadius
	
	def getRadiusScale(self):
		
		"""Returns radius scaling factor.
		"""
		
		return self.radiusScale
	
	def computeDome(self):
		
		r"""Updates zebrafish geometry.
		
		Computes zebrafish geometry as follows:
		
		.. math:: r_{\mathrm{outer}}=\frac{r_{\mathrm{imaging}}^2+h_{\mathrm{imaging}}^2}{-2 h_{\mathrm{imaging}}},\\
		   r_{\mathrm{inner}}=s_{\mathrm{radius}}r_{\mathrm{outer}},\\
		   d_{\mathrm{center}}=\sqrt{r_{\mathrm{inner}}^2-r_{\mathrm{outer}}^2},
		   
		where :math:`r_{\mathrm{outer}}` is the ``outerRadius``, :math:`r_{\mathrm{inner}}` is the ``innerRadius``, :math:`h_{\mathrm{imaging}}` 
		is the ``imagingHeight`` and :math:`r_{\mathrm{imaging}}` is the ``imagingRadius``.
		
		Returns:
			tuple: Tuple containing:
			
				* innerRadius (float): New inner radius.
				* centerDist (float): New distance between centers.
				
		"""
		
		self.outerRadius=(self.imagingRadius**2+self.imagingHeight**2)/(2*(-self.imagingHeight))
		self.innerRadius=self.outerRadius*self.radiusScale
		self.centerDist=np.sqrt(self.innerRadius**2-self.outerRadius**2) 
		return self.innerRadius,self.centerDist
	
	def restoreDefault(self):
		
		"""Restores default values.
		
		Only default value of zebrafish geometry is that ``imagingHeight`` is set to
		be equal to ``sliceHeightPx`` of ``embryo``.
		
		"""
		
		self.imagingHeight=self.embryo.sliceHeightPx
		self.computeDome()
	
	def updateGeoFile(self,debug=False):
		
		"""Updates .geo file of geometry.
		
		Keyword Args:
			debug (bool): Print debugging messages.
			
		"""
		
		pyfrp_gmsh_module.updateDomeGeo(self.fnGeo,self.imagingRadius,self.imagingHeight,self.center,run=False,debug=debug)
		
	def optimalAllROI(self,name='',Id=0,color='b',asMaster=False):	
		
		"""Sets optimal ROI to a :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI`
		with radius ``outerRadius``, center ``center``, covering the whole z-range of 
		geometry.
		
		Keyword Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			color (str): Color that ROI is going to be associated with.
			asMaster (bool): Make new ROI masterROI.
			
		
		"""
		
		return self.embryo.newRadialSliceROI(name,Id,self.getCenter(),self.getOuterRadius(),0,np.inf,False,color=color,asMaster=asMaster)
	
	def getZExtend(self):
		
		"""Overwrites :py:func:`pyfrp.subclasses.pyfrp_geometry.geometry.getZExtend`.
		
		By default, zebrafish geometry is set to range from ``-outerRadius`` to ``0``. 
		"""
		
		z=[-self.outerRadius,0]
		
		return min(z),max(z)
		
	def getXYExtend(self):
		
		"""Overwrites :py:func:`pyfrp.subclasses.pyfrp_geometry.geometry.getXYExtend`.
		
		By default, zebrafish geometry is set to range from ``center[i]-outerRadius`` to ``center[i]+outerRadius``. 
		
		Returns:
			tuple: Tuple containing:
				
				* xmin (float): Minimum x-coordinate.
				* xmax (float): Maximum x-coordinate.
				* ymin (float): Minimum y-coordinate.
				* ymax (float): Maximum y-coordinate.
		
		"""
		
		return self.center[0]-self.outerRadius,self.center[0]+self.outerRadius,self.center[1]-self.outerRadius,self.center[1]+self.outerRadius
	
	def getVolume(self):
		
		r"""Returns volume of geometry.
		
		Volume is computed by:
		
		.. math:: V_{dome}=\frac{\pi}{6}(4 r_{\mathrm{outer}}^3 -  (d_{\mathrm{center}}-r_{\mathrm{inner}})(3r_{\mathrm{outer}}^2+(d_{\mathrm{center}}-r_{\mathrm{inner}})^2))	
		
		Returns:
			float: Volume of zebrafish dome.
		
		"""
		
		Vouter=4./6.*np.pi * self.outerRadius**2
		Vinner=1/6.*np.pi * (self.centerDist-self.innerRadius)*(3*self.outerRadius+(self.centerDist -  self.innerRadius)**2)
		
		return Vouter-Vinner
	
	def genAsOpenscad(self):
		
		"""Generates zebrafish geometry as solid python object.
		
		Useful if geometry is used to be passed to openscad.
		
		Returns:
			solid.solidpython.openscad_object: Solid python object. 
		
		"""
		
		outerBall=solid.translate([self.center[0],self.center[1],-self.outerRadius])(solid.sphere(r=self.outerRadius))
		innerBall=solid.translate([self.center[0],self.center[1],-self.outerRadius-self.centerDist])(solid.sphere(r=self.innerRadius))
	
		return outerBall-innerBall
		
class zebrafishDomeStageQuad(zebrafishDomeStage):
	
	"""Geometry describing a zebrafish embryo in dome stage, reduced to first quadrant.
	
	Inherits from :py:class:`zebrafishDomeStage`. Please refer to its documentation for
	further details.
	
	.. image:: ../imgs/pyfrp_geometry/domequad.png
	
	"""
	
	def __init__(self,embryo,center,imagingRadius,radiusScale=1.1):
		
		mdir=pyfrp_misc_module.getMeshfilesDir()
		zebrafishDomeStage.__init__(self,embryo,center,imagingRadius,radiusScale=1.1)
		geometry.__init__(self,embryo,"zebrafishDomeStageQuad",mdir+"quad_dome.geo",center)
	
	
class cylinder(geometry):
	
	"""Geometry describing a cylinder.
	
	.. image:: ../imgs/pyfrp_geometry/cylinder.png
	
	The crucial geometrical parameters are:
		
		* ``radius``: Radius of the cylinder.
		* ``height``: Height of the cylinder.
		
	"""
	
	def __init__(self,embryo,center,radius,height):
		mdir=pyfrp_misc_module.getMeshfilesDir()
		super(cylinder, self).__init__(embryo,"cylinder",mdir+"cylinder.geo",center)

		self.radius=radius
		self.height=height
		self.updateGeoFile()
		
	def setHeight(self,h):
		
		"""Sets cylinder height.
		
		Args:
			h (float): New height.
		
		Returns:
			float: New height.
		
		"""
		
		self.height=h
		return self.height
	
	def setRadius(self,r):
		
		"""Sets cylinder radius.
		
		Args:
			h (float): New radius.
		
		Returns:
			float: New radius.
		
		"""
		
		self.radius=r
		return self.radius
	
	def getHeight(self):
		
		"""Returns cylinder height.
		
		Returns:
			float: Height.
		
		"""
		
		return self.height
	
	def getRadius(self):
		
		"""Returns cylinder radius.
		
		Returns:
			float: Radius.
		
		"""
		
		return self.radius
	
	def updateGeoFile(self,debug=False):
		
		"""Updates .geo file of geometry.
		
		Keyword Args:
			debug (bool): Print debugging messages.
			
		"""
		
		pyfrp_gmsh_module.updateCylinderGeo(self.fnGeo,self.radius,self.height,self.center,run=False,debug=debug)

	def optimalAllROI(self,name='',Id=0,color='b',asMaster=False,roi=None):	
		
		"""Sets optimal ROI to a :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI`
		with radius ``radius``, center ``center``, covering the whole z-range of 
		geometry.
		
		Keyword Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			color (str): Color that ROI is going to be associated with.
			asMaster (bool): Make new ROI masterROI.
			
		
		"""
		
		return self.embryo.newRadialSliceROI(name,Id,self.getCenter(),self.getRadius(),0.,np.inf,False,color=color,asMaster=asMaster)
	
	def getZExtend(self):
		
		"""Overwrites :py:func:`pyfrp.subclasses.pyfrp_geometry.geometry.getZExtend`.
		
		By default, cylinder geometry is set to range from ``-height`` to ``0``. 
		"""
		z=[-self.height,0]
		return min(z),max(z)
	
	def getXYExtend(self):
		
		"""Overwrites :py:func:`pyfrp.subclasses.pyfrp_geometry.geometry.getXYExtend`.
		
		By default, cylinder geometry is set to range from ``center[i]-radius`` to ``center[i]+radius``. 
		
		Returns:
			tuple: Tuple containing:
				
				* xmin (float): Minimum x-coordinate.
				* xmax (float): Maximum x-coordinate.
				* ymin (float): Minimum y-coordinate.
				* ymax (float): Maximum y-coordinate.
		
		"""
		
		return self.center[0]-self.radius,self.center[0]+self.radius,self.center[1]-self.radius,self.center[1]+self.radius
	
	def genAsOpenscad(self):
		
		"""Generates cylinder geometry as solid python object.
		
		Useful if geometry is used to be passed to openscad.
		
		Returns:
			solid.solidpython.openscad_object: Solid python object. 
		
		"""
		
		z=self.getZExtend()
		cylinder=solid.translate([self.center[0],self.center[1],min(z)])(solid.cylinder(r=self.radius,h=abs(self.height)))
		
		return cylinder
	
class cylinderQuad(cylinder):
	
	"""Geometry describing a cylinder, reduced to first quadrant.
	
	Inherits from :py:class:`cylinder`. Please refer to its documentation for
	further details.
	
	.. image:: ../imgs/pyfrp_geometry/cylinderquad.png
	
	"""
	
	def __init__(self,embryo,center,radius,height):
		
		mdir=pyfrp_misc_module.getMeshfilesDir()
		cylinder.__init__(self,embryo,center,radius,height)
		geometry.__init__(self,embryo,"cylinderQuad",mdir+"quad_cylinder.geo",center)
		
	
class xenopusBall(geometry):
	
	"""Geometry describing a ball.
	
	This geometry is similar to a xenopus in stage 7-10, see http://www.xenbase.org/anatomy/alldev.do.
	
	.. image:: ../imgs/pyfrp_geometry/ball.png
	
	The crucial geometrical parameters are:
		
		* ``radius``: Radius of the ball.
		
	PyFRAP automatically computes these parameters given 
	
		* ``imagingRadius``: Radius of embryo at imaging depth.
		* ``imagingHeight``: Imaging depth.
		
	For details of this computations, see :py:func:`computeBall`.
		
	"""
	
	def __init__(self,embryo,center,imagingRadius):		
		
		mdir=pyfrp_misc_module.getMeshfilesDir()
		super(xenopusBall, self).__init__(embryo,"xenopusBall",mdir+"ball.geo",center)
		
		self.imagingRadius=imagingRadius
		
		self.restoreDefault()
		self.updateGeoFile()
	
	def setImagingRadius(self,r):
		
		"""Sets imaging radius and updates ball geometry.
		
		Args:
			h (float): New radius.
		
		Returns:
			float: New radius.
		
		"""
		
		self.imagingRadius=r
		self.computeBall()
		return self.imagingRadius
	
	def setImagingHeight(self,h):
		
		"""Sets imaging height and updates ball geometry.
		
		Args:
			h (float): New height.
		
		Returns:
			float: New height.
		
		"""
		
		
		self.imagingHeight=h
		self.computeBall()
		return self.imagingHeight
	
	def getImagingRadius(self,r):
		
		"""Returns imaging radius."""
		
		return self.imagingRadius
	
	def getImagingHeight(self,h):
		
		"""Returns imaging height."""
		
		return self.imagingHeight
	
	def computeBall(self):
		
		r"""Computes ball geometry from ``imagingRadius`` and ``imagingHeight``.
		
		Computes ball geometry as follows:
		
		.. math:: r=\frac{r_{\mathrm{imaging}}^2+h_{\mathrm{imaging}}^2}{-2 h_{\mathrm{imaging}}},
		   
		where :math:`r` is the ``radius``, :math:`h_{\mathrm{imaging}}` 
		is the ``imagingHeight`` and :math:`r_{\mathrm{imaging}}` is the ``imagingRadius``.
		
		The center of the ball is set to ``[center[0],center[1],-radius]`` (Only in .geo file).
		
		"""
		
		self.radius=(self.imagingRadius**2+self.imagingHeight**2)/(2*(-self.imagingHeight))
	
	def restoreDefault(self):
		
		"""Restores default values.
		
		Only default value of ball geometry is that ``imagingHeight`` is set to
		be equal to ``sliceHeightPx`` of ``embryo``.
		
		"""
		
		self.imagingHeight=self.embryo.sliceHeightPx
		self.computeBall()
		
	def updateGeoFile(self,debug=False):
		
		"""Updates .geo file of geometry.
		
		Keyword Args:
			debug (bool): Print debugging messages.
			
		"""
		
		pyfrp_gmsh_module.updateBallGeo(self.fnGeo,self.radius,self.center,run=False,debug=debug)
	
	def getRadius(self):
		
		"""Returns ball radius."""
		
		return self.radius
	
	def optimalAllROI(self,name='',Id=0,color='b',asMaster=False,roi=None):	
		
		"""Sets optimal ROI to a :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI`
		with radius ``radius``, center ``center``, covering the whole z-range of 
		geometry.
		
		Keyword Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			color (str): Color that ROI is going to be associated with.
			asMaster (bool): Make new ROI masterROI.
			
		
		"""
		
		return self.embryo.newRadialSliceROI(name,Id,self.getCenter(),self.getRadius(),0.,np.inf,False,color=color,asMaster=asMaster)
	
	def getZExtend(self):
		
		"""Overwrites :py:func:`pyfrp.subclasses.pyfrp_geometry.geometry.getZExtend`.
		
		By default, ball geometry is set to range from ``-2*imagingRadius`` to ``0``. 
		"""
		
		z=[-2*self.imagingRadius,0]
		
		return min(z),max(z)
	
	def getXYExtend(self):
		
		"""Overwrites :py:func:`pyfrp.subclasses.pyfrp_geometry.geometry.getXYExtend`.
		
		By default, ball geometry is set to range from ``center[i]-radius`` to ``center[i]+radius``. 
		
		Returns:
			tuple: Tuple containing:
				
				* xmin (float): Minimum x-coordinate.
				* xmax (float): Maximum x-coordinate.
				* ymin (float): Minimum y-coordinate.
				* ymax (float): Maximum y-coordinate.
		
		"""
		
		return self.center[0]-self.radius,self.center[0]+self.radius,self.center[1]-self.radius,self.center[1]+self.radius
	
class xenopusBallQuad(xenopusBall):
	
	"""Geometry describing a ball, reduced to first quadrant.
	
	Inherits from :py:class:`xenopusBall`. Please refer to its documentation for
	further details.
	
	.. warning:: ``meshfiles/quad_ball.geo`` does not exist yet. 
	
	"""
	
	def __init__(self,embryo,center,imagingRadius):
		
		mdir=pyfrp_misc_module.getMeshfilesDir()
		xenopusBall.__init__(self,embryo,center,imagingRadius)	
		geometry.__init__(self,embryo,"xenopusBallQuad",mdir+"quad_ball.geo",center)	

class cone(geometry):
	
	"""Geometry describing a cut-off cone.
	
	.. image:: ../imgs/pyfrp_geometry/cone.png
	
	The crucial geometrical parameters are:
		
		* ``upperRadius``: Upper radius of cone.
		* ``lowerRadius``: Lower radius of cone.
		* ``height``: Height of cone.
		
	.. note:: Can also be extended to a real cone by setting 
	   ``lowerRadius=0``.
	
	"""
	
	def __init__(self,embryo,center,upperRadius,lowerRadius,height):
		
		mdir=pyfrp_misc_module.getMeshfilesDir()
		super(cone, self).__init__(embryo,"cone",mdir+"cone.geo",center)

		self.upperRadius=upperRadius
		self.lowerRadius=lowerRadius
		
		self.height=height
		
		self.updateGeoFile()
		
	def setHeight(self,h):
		
		"""Sets cone height.
		
		Args:
			h (float): New height.
		
		Returns:
			float: New height.
		
		"""
		
		self.height=h
		return self.height
	
	def setLowerRadius(self,r):
		
		"""Sets cone lower radius.
		
		Args:
			h (float): New lower radius.
		
		Returns:
			float: New lower radius.
		
		"""
		
		self.lowerRadius=r
		return self.lowerRadius
	
	def getLowerRadius(self):
		
		"""Returns lower radius of cone."""
		
		return self.lowerRadius
	
	def setUpperRadius(self,r):
		
		"""Sets cone upper radius.
		
		Args:
			h (float): New upper radius.
		
		Returns:
			float: New upper radius.
		
		"""
		
		self.upperRadius=r
		return self.upperRadius
	
	def getUpperRadius(self):
		
		"""Returns upper radius of cone."""
		
		return self.upperRadius
	
	def getHeight(self):
		
		"""Returns cone radius."""
		
		return self.height
	
	def computeSliceHeightFromRadius(self,radius):
		
		r"""Returns the slice height given a slice radius.
		
		Slice height is computed by
		
		.. math:: s(r) = \frac{h}{l-u} (r-u)
		
		where :math:`l,u` are lower and upper radius respectively and :math:`h` is cone height.
		
		Args:
			radius (float): Slice radius.
			
		Returns:
			float: Slice height.
		
		"""
		
		sliceHeight=self.height/(self.lowerRadius-self.upperRadius)*(radius-self.upperRadius) 
		return sliceHeight
	
	def computeRadiusFromSliceHeight(self,height):
		
		r"""Returns the slice radius given a slice height.
		
		Slice radius is computed by
		
		.. math:: r(s) = \frac{l-u}{h} s +u
		
		where :math:`l,u` are lower and upper radius respectively and :math:`h` is cone height.
		
		Args:
			height (float): Slice height.
			
		Returns:
			float: Slice radius.
		
		"""
		
		radius=(self.upperRadius-self.lowerRadius)/self.height*height+self.upperRadius
		return radius
	
	def updateGeoFile(self,debug=False):
		
		"""Updates .geo file of geometry.
		
		Keyword Args:
			debug (bool): Print debugging messages.
			
		"""
		
		pyfrp_gmsh_module.updateConeGeo(self.fnGeo,self.upperRadius,self.lowerRadius,abs(self.height),self.center,run=False,debug=debug)

	def optimalAllROI(self,name='',Id=0,color='b',asMaster=False,roi=None):	
		
		"""Sets optimal ROI to a :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI`
		with radius ``upperRadius``, center ``center``, covering the whole z-range of 
		geometry.
		
		Keyword Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			color (str): Color that ROI is going to be associated with.
			asMaster (bool): Make new ROI masterROI.
			
		
		"""
		
		return self.embryo.newRadialSliceROI(name,Id,self.getCenter(),self.getUpperRadius(),0.,np.inf,False,color=color,asMaster=asMaster)
	
	def getZExtend(self):
		
		"""Overwrites :py:func:`pyfrp.subclasses.pyfrp_geometry.geometry.getZExtend`.
		
		By default, cone geometry is set to range from ``-height`` to ``0``. 
		"""
		
		z=[-self.height,0]
		
		return min(z),max(z)
		
	def getXYExtend(self):
		
		"""Overwrites :py:func:`pyfrp.subclasses.pyfrp_geometry.geometry.getXYExtend`.
		
		By default, cone geometry is set to range from ``center[i]-max([self.upperRadius,self.lowerRadius])`` 
		to ``center[i]+max([self.upperRadius,self.lowerRadius])``. 
		
		Returns:
			tuple: Tuple containing:
				
				* xmin (float): Minimum x-coordinate.
				* xmax (float): Maximum x-coordinate.
				* ymin (float): Minimum y-coordinate.
				* ymax (float): Maximum y-coordinate.
		
		"""
		
		maxRadius=max([self.upperRadius,self.lowerRadius])
		
		return self.center[0]-maxRadius,self.center[0]+maxRadius,self.center[1]-maxRadius,self.center[1]+maxRadius
	
	def genAsOpenscad(self):
		
		"""Generates cone geometry as solid python object.
		
		Useful if geometry is used to be passed to openscad.
		
		Returns:
			solid.solidpython.openscad_object: Solid python object. 
		
		"""
		
		z=self.getZExtend()
		cone=solid.translate([self.center[0],self.center[1],min(z)])(solid.cylinder(r1=self.lowerRadius,h=self.height,r2=self.upperRadius))
		
		return cone
	
class custom(geometry):
	
	"""Custom geometry class for custom geometry configurations.

	"""
	
	def __init__(self,embryo,center,fnGeo,dim=3):
		
		geometry.__init__(self,embryo,"custom",fnGeo,center,dim=dim)
	
	


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFDAP.

#PyFDAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#Module containing classes to emulate Python terminal in PyQT GUI for PyFRAP.
#Modified from http://stackoverflow.com/questions/2758159/how-to-embed-a-python-interpreter-in-a-pyqt-widget/8219536#8219536

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

import os
import re
import sys
import code
import colorama

from PyQt4 import QtGui, QtCore


#===========================================================================================================================================================================
#Module classes
#===========================================================================================================================================================================

class PyInterp(QtGui.QTextEdit):

	class InteractiveInterpreter(code.InteractiveInterpreter):

		def __init__(self, locals):
			code.InteractiveInterpreter.__init__(self, locals)

		def runIt(self, command):
			code.InteractiveInterpreter.runsource(self, command)


	def __init__(self,  parent,redirect=True):
		super(PyInterp,  self).__init__(parent)
		
		if redirect:
			sys.stdout              = self
			sys.stderr              = self
			
		self.refreshMarker      = False # to change back to >>> from ...
		self.multiLine          = False # code spans more than one line
		self.command            = ''    # command to be ran
		
		self.history            = []    # list of commands entered
		self.historyIndex       = -1
		self.interpreterLocals  = {}
		
		self.colorKeys          = {}
		
		#self.greenKey           = '[32m'
		
		
		# setting the color for bg and text
		palette = QtGui.QPalette()
		palette.setColor(QtGui.QPalette.Base, QtGui.QColor(255, 255, 255))
		palette.setColor(QtGui.QPalette.Text, QtGui.QColor(0, 0, 0))
		self.setPalette(palette)
		self.setFont(QtGui.QFont('Courier', 12))
	
		# initilize interpreter with self locals
		self.initInterpreter(locals())
		
		#Import some standard classes that might be useful
		#self.interpreter.runIt('import pyfrp_img_module as img')
		#self.interpreter.runIt('import pyfrp_fit_module as fit')
		#self.interpreter.runIt('import pyfrp_misc_module as misc')
		self.interpreter.runIt('import numpy as np')
		self.interpreter.runIt('import colorama')
		self.interpreter.runIt('from pyfrp.modules.pyfrp_term_module import *')
		
		self.initColorKeys()
		
		#Print banner and initialize
		self.printBanner()    
		self.marker()                   # make the >>> or ... marker     
	
	def printBanner(self):
		self.write(sys.version)
		self.write(' on ' + sys.platform + '\n')
		self.write('PyQt4 ' + QtCore.PYQT_VERSION_STR + '\n')
        
        
		#self.write('PyFrap Version' + pyfrp_version + '\n')
		#self.write("---------------------------------------"+ '\n')
		#msg = 'Type !hist for a history view and !hist(n) history index recall'
		#self.write(msg + '\n')
		#msg2 = 'To access local variables, type pyfdp.variable. For example, to access the currently selected molecule, type pyfdp.curr_mol. For more details, see the documentation.'
		#self.write(msg2 + '\n')
		#msg3 = 'To access functions of the pyfdp modules, type modulename.function(). Available modulenames are "fit", "img" and "misc".'
		#self.write(msg3 + '\n')
		#self.write("---------------------------------------" + '\n')

	def marker(self):
		if self.multiLine:
			self.insertPlainText('... ')
		else:
			self.insertPlainText('>>> ')

	def initInterpreter(self, interpreterLocals=None):
     
		if interpreterLocals:
			# when we pass in locals, we don't want it to be named "self"
			# so we rename it with the name of the class that did the passing
			# and reinsert the locals back into the interpreter dictionary
			
			selfName = interpreterLocals['self'].__class__.__name__
			
			interpreterLocalVars = interpreterLocals.pop('self')
			self.interpreterLocals[selfName] = interpreterLocalVars
			
		else:
			self.interpreterLocals = interpreterLocals
		self.interpreter = self.InteractiveInterpreter(self.interpreterLocals)
	
	def initColorKeys(self):
		self.colorKeys['[32m']='green'
		self.colorKeys['[39m']='black'
		self.colorKeys['[33m']='orange'
		self.colorKeys['[31m']='red'
		return self.colorKeys
		
	def updateInterpreterLocals(self, newLocals):
		className = newLocals.__class__.__name__
		self.interpreterLocals[className] = newLocals

	def write(self, line):
		colors,strings=self.findColors(line,self.colorKeys)
	
		if len(colors)==0:
			self.insertPlainText(line)
		else:
			txt=""
		
			for i in range(len(colors)):
				txt=txt+self.makeColoredText(strings[i],colors[i])
			self.insertHtml(txt)	
		
		self.ensureCursorVisible()
	
	def makeColoredText(self,txt,color):
		coloredText=redText = "<span style=\" color:"+color+"; \" >" + txt + "</span>" 
		return coloredText
	
	def findColors(self,line,colorDic):
	
		found=0
		
		colors=[]
		strings=[]
		
		while found!=-1:
			
			fs=[]
			cs=[]
			for key in colorDic.keys(): 
				ind=line.find(key,found)
				if ind!=-1:
					fs.append(ind)
				else: 
					fs.append(str(ind))
			
			foundOld=found
			found=min(fs)
				
			if len(colors)==0 and found!='-1':
				colors.append(colorDic.values()[fs.index(found)])
			elif len(colors)>0:
				offset=len(colorDic.keys()[fs.index(found)])-1
				if found=='-1':
					strings.append(line[foundOld+offset:])
				else:
					strings.append(line[foundOld+offset:found].strip('\x1b'))
					colors.append(colorDic.values()[fs.index(found)])
				
			found=int(found)
			if found!=-1:
				found=found+1
		
		return colors,strings
					
	def clearCurrentBlock(self):
		# block being current row
		length = len(self.document().lastBlock().text()[4:])
		if length == 0:
			return None
		else:
			# should have a better way of doing this but I can't find it
			[self.textCursor().deletePreviousChar() for x in xrange(length)]
		return True
		
    
	def recallHistory(self):
		# used when using the arrow keys to scroll through history
		self.clearCurrentBlock()
		if self.historyIndex <> -1:
			self.insertPlainText(self.history[self.historyIndex])
		return True

	def customCommands(self, command):

		if command == '!hist': # display history
			self.append('') # move down one line
			# vars that are in the command are prefixed with ____CC and deleted
			# once the command is done so they don't show up in dir()
			backup = self.interpreterLocals.copy()
			history = self.history[:]
			history.reverse()
			for i, x in enumerate(history):
				iSize = len(str(i))
				delta = len(str(len(history))) - iSize
				line = line  = ' ' * delta + '%i: %s' % (i, x) + '\n'
				self.write(line)
		
			self.updateInterpreterLocals(backup)
			self.marker()
			return True

		if re.match('!hist\(\d+\)', command): # recall command from history
			backup = self.interpreterLocals.copy()
			history = self.history[:]
			history.reverse()
			index = int(command[6:-1])
			self.clearCurrentBlock()
			command = history[index]
			if command[-1] == ':':
				self.multiLine = True
			self.write(command)
			self.updateInterpreterLocals(backup)
			return True

		return False

	def keyPressEvent(self, event):

		if event.key() == QtCore.Qt.Key_Escape:
			# proper exit
			self.interpreter.runIt('exit()')

		if event.key() == QtCore.Qt.Key_Down:
			if self.historyIndex == len(self.history):
				self.historyIndex -= 1
			try:
				if self.historyIndex > -1:
					self.historyIndex -= 1
					self.recallHistory()
				else:
					self.clearCurrentBlock()
			except:
				pass
			return None

		if event.key() == QtCore.Qt.Key_Up:
			try:
				if len(self.history) - 1 > self.historyIndex:
					self.historyIndex += 1
					self.recallHistory()
				else:
					self.historyIndex = len(self.history)
			except:
				pass
			return None

		if event.key() == QtCore.Qt.Key_Home:
			# set cursor to position 4 in current block. 4 because that's where
			# the marker stops
			blockLength = len(self.document().lastBlock().text()[4:])
			lineLength  = len(self.document().toPlainText())
			position = lineLength - blockLength
			textCursor  = self.textCursor()
			textCursor.setPosition(position)
			self.setTextCursor(textCursor)
			return None

		if event.key() in [QtCore.Qt.Key_Left, QtCore.Qt.Key_Backspace]:
			# don't allow deletion of marker
			if self.textCursor().positionInBlock() == 4:
				return None

		if event.key() in [QtCore.Qt.Key_Return, QtCore.Qt.Key_Enter]:
			# set cursor to end of line to avoid line splitting
			textCursor = self.textCursor()
			position   = len(self.document().toPlainText())
			textCursor.setPosition(position)
			self.setTextCursor(textCursor)

			line = str(self.document().lastBlock().text())[4:] # remove marker
			line.rstrip()
			self.historyIndex = -1

			if self.customCommands(line):
				return None
			else:
				try:
					line[-1]
					self.haveLine = True
					if line[-1] == ':':
						self.multiLine = True
					self.history.insert(0, line)
				except:
					self.haveLine = False

				if self.haveLine and self.multiLine: # multi line command
					self.command += line + '\n' # + command and line
					self.append('') # move down one line
					self.marker() # handle marker style
					return None

				if self.haveLine and not self.multiLine: # one line command
					self.command = line # line is the command
					self.append('') # move down one line
					self.interpreter.runIt(self.command)
					self.command = '' # clear command
					self.marker() # handle marker style
					return None

				if self.multiLine and not self.haveLine: #  multi line done
					self.append('') # move down one line
					self.interpreter.runIt(self.command)
					self.command = '' # clear command
					self.multiLine = False # back to single line
					self.marker() # handle marker style
					return None

				if not self.haveLine and not self.multiLine: # just enter
					self.append('')
					self.marker()
					return None
				return None
			
		# allow all other key events
		super(PyInterp, self).keyPressEvent(event)


#Import setuptools
from setuptools import setup

#We need those two for doing some file permission magic later
import os
import platform
import shutil

#Overwrite setuptools install to be able to set file permissions
from setuptools.command.install import install
from distutils import log 
from setuptools.command.install_scripts import install_scripts

#Import option parser so we can parse in options
import sys

# Define version
version='1.1.7'

def getOptions():
	
	"""Checks options given to script.
	
	If --fiji is in sys.argv, will set dFiji=1. \n
	If --gmsh is in sys.argv, will set dGmsh=1.
	If --silent is in sys.argv, will set silent=1.
	
	
	
	Note: Makes dGmsh and dFiji global: Not nice but seems easiest 
	way to get options into OverrideInstall.
	"""
	
	global dGmsh
	global dFiji
	global silent
	
	dFiji=getOpt("--fiji")
	dGmsh=getOpt("--noGmsh",default=0)
	silent=getOpt("--silent")
	
def getOpt(optStr,default=0):	
	
	"""Checks if optStr is in sys.argv. If this is the case,
	returns 1 and removes it form sys.argv so setup.py will not crash,
	otherwise returns default.
	"""
	
	if optStr in sys.argv:
		opt=1
		sys.argv.remove(optStr)
	else:
		opt=default
	return opt	

#Get Options
if __name__ == '__main__':
	getOptions()

class OverrideInstall(install):
	
	"""Override class subclassing install class from setuptools.
	
	The Main purpose of this class is to give more possibilities when installing PyFRAP, such as:
	
		* Download Gmsh and enter it automatically into path spec file
		* Download Fiji and enter it automatically into path spec file
		* Set ownership of data files so that even PyFRAP gets installed as superuser,
		users will be able to use its full capacities.
	
	Idea taken from http://stackoverflow.com/questions/5932804/set-file-permission-in-setup-py-file (thanks a bunch!)
	
	"""
	
	def initOptions(self):
		"""Parses options into override class.
		"""
		self.dFiji=bool(dFiji)
		self.dGmsh=not bool(dGmsh)
		self.silent=bool(silent)
		
		#Define pathFile
		self.pathFile='paths'
			
	def run(self):
		
		"""Runs install. 
		
		"""
		
		self.initOptions()
		
		#Try to download gmsh
		if self.dGmsh:
			self.downloadGmsh()
		else:
			self.gmshDownloaded=False
		
		#Try to download fiji
		if self.dFiji:
			self.downloadFiji()
		else:
			self.fijiDownloaded=False
		
		#Run setuptools install
		install.run(self) 
		
		#Print log info
		if not self.silent:
			log.info("Overriding setuptools mode of scripts ...")
		
		#Add Data and edit file permissions
		self.addData()
			
	def addData(self):
		
		"""Adds Datafiles to PyFRAP installation. 
		
		Makes sure that $USER has proper read/write/execute rights. Note that for Windows it will change rights,
		since it is not necesary. \n 
		Also makes sure that gmsh/Fiji bin ins properly linked.
		"""
		
		if not self.silent:
			log.info("in add data")
		
		uid,gid,mode=self.getPermDetails()
		
		#Overwrite file permissions
		for filepath in self.get_outputs():
			
			#log.info("Copying files.")
			
			if platform.system() not in ["Windows"]:
			
				if "meshfiles" in filepath or "configurations" in filepath or "executables" in filepath:
					
					#Change permissions for folder containing files
					folderpath=os.path.dirname(os.path.realpath(filepath))
					self.changePermissions(folderpath,uid,gid,mode)
					
					#Change permissions of file
					self.changePermissions(filepath,uid,gid,mode)
					
					#Make some more necessary data folders
					if folderpath.endswith("meshfiles"):
						self.makeAdditionalDataFolders(folderpath,"field",uid,gid,mode)
						self.makeAdditionalDataFolders(folderpath,"field/custom",uid,gid,mode)
						
					if folderpath.endswith("configurations"):
						self.makeAdditionalDataFolders(folderpath,"macros",uid,gid,mode)
			
			#log.info("Adding executables to path file")
			
			#Add gmsh into paths.default if download was successful
			if self.pathFile == os.path.basename(filepath):
				
				if self.gmshDownloaded:
					self.setGmshPath(filepath)
				if self.fijiDownloaded:
					self.setFijiPath(filepath)
				if platform.system() not in ["Windows"]:
					folderpath=os.path.dirname(os.path.realpath(filepath))
					self.changePermissions(folderpath,uid,gid,mode)
					self.changePermissions(filepath,uid,gid,mode)

	
	def getPermDetails(self):
		
		"""Returns the permission details used to change permissions.
		"""
		
		if platform.system() not in ["Windows"]:
			import pwd
			
			#Grab user ID and group ID of actual use
			try:
				uid=pwd.getpwnam(os.getlogin())[2]
				gid=pwd.getpwnam(os.getlogin())[3]
			
			except: 
				if not self.silent:
					log.info("Was not able to retrieve UID via os.getlogin, using os.getuid instead.")
				uid=os.getuid()
				gid=os.getgid()
				
			#Mode for files (everyone can read/write/execute. This is somewhat an overkill, but 0666 seems somehow not to work.)
			mode=0777
			
			return uid,gid,mode
			
		return	0,0,0
	
	def cleanUpExe(self,fnDL,folderFn,filesBefore,exePath):		
		
		"""Moves it to executables directory and cleans up afterwards. 
		
		"""
		
		#Copy file to pyfrp/executables/
		try:
			shutil.rmtree(exePath)
		except:
			pass
		
		
		
		if os.path.isdir(folderFn):
			shutil.copytree(folderFn+"/",exePath)
		else:
			try:
				os.mkdir(os.path.dirname(exePath))
			except:
				print "Was not able to create folder " + os.path.dirname(exePath)
			shutil.copy(folderFn,exePath)
				
		#Remove downloaded files
		os.remove(fnDL)
		
		#Get fileList before
		filesAfter=os.listdir('.')
		
		self.cleanDiff(filesBefore,filesAfter)
		
	def cleanDiff(self,filesBefore,filesAfter):
		
		#Difference between files
		filesDiff=list(set(filesAfter)-set(filesBefore))
		
		for fn in filesDiff:
				
			try:
				if os.path.isdir(fn):
					shutil.rmtree(fn)
				else:
					os.remove(fn)
			except:
				print "cleanDiff report: Was not able to delete file:" + fn
				
	def downloadGmsh(self):
		
		"""Downloads Gmsh, moves it to executables directory and cleans up afterwards. 
		
		Note that this will only work if *wget* is installed. 
		"""
		
		#Define gmshVersion (might need to update this line once in a while)
		gmshVersion='2.14.0'
		
		#Flag to see if gmsh DL went through
		self.gmshDownloaded=False
		
		self.makeExeFolder()
		
		#Get fileList before
		filesBefore=os.listdir('.')
		
		#Try to import wget
		try: 
			import wget
			
			#Get Architecture
			arch=platform.architecture()[0].replace('bit','')
			
			if platform.system() in ["Windows"]:
				fnDL,folderFn=self.downloadGmshWin(arch,gmshVersion)
				
			elif platform.system() in ["Linux"]:
				fnDL,folderFn=self.downloadGmshLinux(arch,gmshVersion)
				
			elif platform.system() in ["Darwin"]:
				fnDL,folderFn=self.downloadGmshOSX(arch,gmshVersion)
			
			#Remove files
			self.cleanUpExe(fnDL,folderFn,filesBefore,'pyfrp/executables/gmsh/')
			
			uid,gid,mode=self.getPermDetails()
			if platform.system() not in ["Windows"]:
				self.changePermissions(self.gmshPath,uid,gid,mode)
			
			self.addPathToWinPATHs(self.gmshPath)
			
			log.info("Installed gmsh to "+ self.gmshPath)
			
			#Set Flag=True
			self.gmshDownloaded=True
			
		except ImportError:
			log.info("Cannot find wget, will not be downloading gmsh. You will need to install it later manually")	
	
	#def downloadOpenscad(self):
		
		#"""Downloads openscad, moves it to executables directory and cleans up afterwards. 
		
		#Note that this will only work if *wget* is installed. 
		#"""
			
		#http://files.openscad.org/OpenSCAD-2015.03-2-x86-64.zip	
		#http://files.openscad.org/OpenSCAD-2015.03-3.dmg
		#http://files.openscad.org/openscad-2014.03.x86-64.tar.gz
		
		##Flag to see if gmsh DL went through
		#self.openscadDownloaded=False
		
		#self.makeExeFolder()
		
		##Get fileList before
		#filesBefore=os.listdir('.')
		
		##Try to import wget
		#try: 
			#import wget
			
			##Get Architecture
			#arch=platform.architecture()[0].replace('bit','')
			
			#if platform.system() in ["Windows"]:
				#fnDL,folderFn=self.downloadOpenscadWin(arch,openscadVersion)
				
			#elif platform.system() in ["Linux"]:
				#fnDL,folderFn=self.downloadOpenscadLinux(arch,openscadVersion)
				
			#elif platform.system() in ["Darwin"]:
				#fnDL,folderFn=self.downloadOpenscadOSX(arch,openscadVersion)
			
			##Remove files
			#self.cleanUpExe(fnDL,folderFn,filesBefore,'pyfrp/executables/openscad/')
			
			#uid,gid,mode=self.getPermDetails()
			#if platform.system() not in ["Windows"]:
				#self.changePermissions(self.openscadPath,uid,gid,mode)
			
			#self.addPathToWinPATHs(self.openscadPath)
			
			#log.info("Installed openscad to "+ self.openscadPath)
			
			##Set Flag=True
			#self.openscadDownloaded=True
			
		#except ImportError:
			#log.info("Cannot find wget, will not be downloading gmsh. You will need to install it later manually")	
				
	
	#def downloadOpenscadWin(self,arch,gmshVersion):
		
		#"""Downloads Gmsh from Gmsh website for Windows
		
		#Args:
			#arch (str): System architecture, e.g. 64/32.
			#gmshVersion (str): gmshVersion String, e.g. 2.12.0 .
			
		#Returns:
			#tuple: Tuple containing:
			
				#* fnDL (str): Donwload filename
				#* folderFn (str): Filename of extracted download files
			
		#"""
		
		##Download Gmsh
		#url='http://gmsh.info/bin/Windows/gmsh-'+gmshVersion+'-Windows'+arch+'.zip'
		#folderFn, fnDL=self.downloadFileIfNotExist(url)
		
		##Decompress
		#import zipfile 
		#with zipfile.ZipFile(folderFn) as zf:
			#zf.extractall()
			
		#folderFn='gmsh-'+gmshVersion+'-Windows'	
		
		#self.gmshPath='executables/gmsh/bin/gmsh.exe'
		
		#return fnDL,folderFn
	
	def downloadFileIfNotExist(self,url):
		
		"""Downloads URL if file does not already exist.
		
		Args:
			url (str): URL to download.
			
		Returns:
			tuple: Tuple containing:
			
				* fnDL (str): Donwload filename
				* folderFn (str): Filename of extracted download files
			
		"""
		
		import wget
		
		cwd=os.getcwd()+"/"
		
		if not os.path.exists(cwd+os.path.basename(url)):
			print cwd+os.path.basename(url) +" does not exist, will download it."
			folderFn=wget.download(url)
		else:
			print cwd+os.path.basename(url) +" alreay exists, will not download."
			folderFn=os.path.basename(url)
		fnDL=str(folderFn)
		print
		
		return folderFn, fnDL
			
	def downloadGmshWin(self,arch,gmshVersion):
		
		"""Downloads Gmsh from Gmsh website for Windows
		
		Args:
			arch (str): System architecture, e.g. 64/32.
			gmshVersion (str): gmshVersion String, e.g. 2.12.0 .
			
		Returns:
			tuple: Tuple containing:
			
				* fnDL (str): Donwload filename
				* folderFn (str): Filename of extracted download files
			
		"""
		
		#Download Gmsh
		url='http://gmsh.info/bin/Windows/gmsh-'+gmshVersion+'-Windows'+arch+'.zip'
		folderFn, fnDL=self.downloadFileIfNotExist(url)
		
		#Decompress
		import zipfile 
		with zipfile.ZipFile(folderFn) as zf:
			zf.extractall()
			
		folderFn='gmsh-'+gmshVersion+'-Windows'	
		
		self.gmshPath='executables/gmsh/bin/gmsh.exe'
		
		return fnDL,folderFn
		
	def downloadGmshOSX(self,arch,gmshVersion):
		
		"""Downloads Gmsh from Gmsh website for OSX.
		
		Args:
			arch (str): System architecture, e.g. 64/32.
			gmshVersion (str): gmshVersion String, e.g. 2.12.0 .
		Returns:
			tuple: Tuple containing:
			
				* fnDL (str): Donwload filename
				* folderFn (str): Filename of extracted download files
			
		"""
		
		
		
		#Download Gmsh (if file isn't there yet)
		url='http://gmsh.info/bin/MacOSX/gmsh-'+gmshVersion+'-MacOSX'+'.dmg'
		folderFn, fnDL=self.downloadFileIfNotExist(url)
		
		#Mount dmg file (Here the user need to read through LICENSE, don't know how to fix this)
		print "executing: ", 'hdiutil attach '+folderFn 
		os.system('hdiutil attach '+folderFn)
		folderFn=folderFn.replace('.dmg','')
		
		#try:
			#os.mkdir(folderFn)
		#except OSError:
			#pass
		
		cwd=os.getcwd()
		
		#Copy gmsh executable to cwd 
		#Note: It seems to vary where gmsh executable is in mounted dmg file, hence we 
		#just have to try out, take the one that actually worked and remember it
		rets=[]
		possFiles=["bin/","share/","gmsh"]
		
		rets.append(os.system('cp -rv /Volumes/'+folderFn+'/Gmsh.app/Contents/MacOS/bin/ '+ cwd))
		rets.append(os.system('cp -rv /Volumes/'+folderFn+'/Gmsh.app/Contents/MacOS/share/ '+ cwd))
		rets.append(os.system('cp -rv /Volumes/'+folderFn+'/Gmsh.app/Contents/MacOS/gmsh '+ cwd))
		
		fnWorked=possFiles[rets.index(0)]
		
		#Unmount gmsh
		os.system('hdiutil detach /Volumes/'+folderFn+'/')
		
		#Build filename of acutally copied file
		folderFn=cwd+"/"+fnWorked
		
		self.gmshPath='executables/gmsh/./gmsh'
		
		return fnDL,folderFn
		
	def downloadGmshLinux(self,arch,gmshVersion):
		
		"""Downloads Gmsh from Gmsh website for Linux.
		
		Args:
			arch (str): System architecture, e.g. 64/32.
			gmshVersion (str): gmshVersion String, e.g. 2.12.0 .
		Returns:
			tuple: Tuple containing:
			
				* fnDL (str): Donwload filename
				* folderFn (str): Filename of extracted download files
			
		"""
	
		#Download Gmsh
		url='http://gmsh.info/bin/Linux/gmsh-'+gmshVersion+'-Linux'+arch+'.tgz'
		folderFn, fnDL=self.downloadFileIfNotExist(url)
		
		#Decompress
		import tarfile
		with tarfile.open(folderFn,mode='r:gz') as zf:
			zf.extractall()
		
		folderFn='gmsh-'+gmshVersion+'-Linux'
		
		self.gmshPath='executables/gmsh/bin/./gmsh'
		
		return fnDL,folderFn	
	
	def makeExeFolder(self):
		
		#Make executables folder if it doesn't exist yet
		try:
			os.mkdir('pyfrp/executables')	
		except OSError:
			log.info('Was not able to create directory pyfrp/executables')
	
	def addPathToWinPATHs(self,path):
	
		"""Adds a path to Windows' PATH list. 
		
		.. note:: Only adds path if file exits.
		
		.. note:: You will need to restart the terminal to 
		be sure that the change has any effect.
		
		Args:
			path (str): Path to be added.
			
		Returns:
			bool: True if successful.
		"""
		
		if platform.system() not in ["Windows"]:
			print "OS is not Windows, won't set path"
			return False
		
		if path in os.environ['PATH']:
			print "Path is already in PATH, won't set path"
			return False
		
		if os.path.exists(path):
			os.system("set PATH=%PATH%;"+path)
			return True
		else:
			print path + " does not exist, won't set path"
			return False
	
	def downloadFiji(self):
		
		"""Downloads Gmsh, moves it to executables directory and cleans up afterwards. 
		
		Note that this will only work if *wget* is installed. 
		"""
		
		#Flag to see if gmsh DL went through
		self.fijiDownloaded=False
		
		self.makeExeFolder()
		
		#Get fileList before
		filesBefore=os.listdir('.')
		
		#Try to import wget
		try: 
			import wget
			
			#Get Architecture
			arch=platform.architecture()[0].replace('bit','')
			
			if platform.system() in ["Windows"]:
				fnDL,folderFn=self.downloadFijiWin(arch)
				
			elif platform.system() in ["Linux"]:
				fnDL,folderFn=self.downloadFijiLinux(arch)
				
			elif platform.system() in ["Darwin"]:
				fnDL,folderFn=self.downloadFijiOSX(arch)
			
			#Remove files
			self.cleanUpExe(fnDL,folderFn,filesBefore,'pyfrp/executables/Fiji.app/')
			
			uid,gid,mode=self.getPermDetails()
			if platform.system() not in ["Windows"]:
				self.changePermissions(self.fijiPath,uid,gid,mode)
			
			self.addPathToWinPATHs(self.fijiPath)
			
			log.info("Installed Fiji to "+ self.fijiPath)
			
			#Set Flag=True
			self.fijiDownloaded=True
			
		except ImportError:
			log.info("Cannot find wget, will not be downloading fiji. You will need to install it later manually")	
	
	def downloadFijiLinux(self,arch):
		
		"""Downloads Fiji from Fiji website for Linux.
		
		Args:
			arch (str): System architecture, e.g. 64/32.
		
		Returns:
			tuple: Tuple containing:
			
				* fnDL (str): Donwload filename
				* folderFn (str): Filename of extracted download files
			
		"""
		
		import wget
		
		#Download Fiji
		url='http://downloads.imagej.net/fiji/latest/fiji-linux'+arch+'.zip'
		
		folderFn=wget.download(url)
		fnDL=str(folderFn)
		print
		
		#Decompress
		import zipfile 
		with zipfile.ZipFile(folderFn) as zf:
			zf.extractall()
		
		
		folderFn='Fiji.app'
		
		self.fijiPath='executables/Fiji.app/./ImageJ-linux64'
		
		return fnDL,folderFn	

	def downloadFijiWin(self,arch):
		
		"""Downloads Fiji from Fiji website for Windows.
		
		Args:
			arch (str): System architecture, e.g. 64/32.
		
		Returns:
			tuple: Tuple containing:
			
				* fnDL (str): Donwload filename
				* folderFn (str): Filename of extracted download files
			
		"""
		
		import wget
		
		#Download fiji
		url='http://downloads.imagej.net/fiji/latest/fiji-win'+arch+'.zip'
		
		folderFn=wget.download(url)
		fnDL=str(folderFn)
		print
		
		#Decompress
		import zipfile 
		with zipfile.ZipFile(folderFn) as zf:
			zf.extractall()
		
		folderFn='Fiji.app'
		
		self.fijiPath='executables/Fiji.app/ImageJ-linux64.exe'
		
		return fnDL,folderFn	
	
	def downloadFijiOSX(self,arch):
		
		"""Downloads Fiji from Fiji website for OSX.
		
		Returns:
			tuple: Tuple containing:
			
				* fnDL (str): Donwload filename
				* folderFn (str): Filename of extracted download files
			
		"""
		
		import wget
		
		#Download fiji
		url='http://downloads.imagej.net/fiji/latest/fiji-macosx.dmg'
		
		folderFn=wget.download(url)
		fnDL=str(folderFn)
		print
	
		
		#Mount dmg file 
		os.system('hdiutil attach '+folderFn)
		
		cwd=os.getcwd()
		
		#Copy fiji executable to cwd
		os.system('cp -rv /Volumes/Fiji/Fiji.app '+ cwd)
		
		#Unmount gmsh
		os.system('hdiutil detach /Volumes/Fiji')
	
		folderFn='Fiji.app'
		
		self.fijiPath='executables/Fiji.app/Contents/MacOS/./ImageJ-macosx'
		
		return fnDL,folderFn	
	
	def setExePath(self,fn,identifier,exePath):
		
		"""Enters executable path into path spec file.
		
		Args:
			fn (str): Path to gmsh executable.
			identifier (str): Identifier in spec file.
			exePath (str): Path to exe file
			
		"""
		
		#Make backup of default path file
		shutil.copy(fn,fn+'_backup')
		
		#Get filepath to PyFRAP
		fnPyfrp=fn.split('configurations')[0]
		
		#Open file and enter new gmsh bin
		with open(fn,'rb') as fPath:
			with open(fn+"_new",'wb') as fPathNew:
				for line in fPath:
					if line.strip().startswith(identifier):
						ident,path=line.split('=')
						path=path.strip()
						lineNew=ident+"="+fnPyfrp+exePath
						fPathNew.write(lineNew+'\n')
					else:
						fPathNew.write(line)
			
		#Rename file
		shutil.move(fn+'_new',fn)
		
	def setGmshPath(self,fn):
		
		"""Enters gmsh executable path into path spec file.
		
		Args:
			fn (str): Path to gmsh executable.
			
		"""
		
		self.setExePath(fn,'gmshBin',self.gmshPath)
	
	def setFijiPath(self,fn):
		
		"""Enters fiji executable path into path spec file.
		
		Args:
			fn (str): Path to fiji executable.
			
		"""
		
		self.setExePath(fn,'fijiBin',self.fijiPath)
	
	def changePermissions(self,filepath,uid,gid,mode):
		
		"""Sets File Permissions.
		
		Args:
			filepath (str): Path to file.
			uid (int): user ID.
			gid (int): group ID.
			mode (int): Permission mode.
		
		Returns:
			bool: True if success
		
		"""
		
		ret=True
		try:
			os.chown(filepath, uid, gid)
			if not self.silent:
				log.info("Changing ownership of %s to uid:%s gid %s" %(filepath, uid, gid))
		except:
			if not self.silent:
				log.info("Was not able to change ownership of file %s" %(filepath))
			ret=False

		try:
			if not self.silent:
				log.info("Changing permissions of %s to %s" %(filepath, oct(mode)))
			os.chmod(filepath, mode)
		except:	
			if not self.silent:
				log.info("Was not able to change file permissions of file %s" %(filepath))
			ret=False
		return ret
		
	def makeAdditionalDataFolders(self,folder,fn,uid,gid,mode):
		
		"""Tries to generate additional data folders.
		
		Args:
			folder (str): Path to containing folder.
			fn (str): New folder name
			uid (int): user ID.
			gid (int): group ID.
			mode (int): Permission mode.
		
		Returns:
			bool: True if success
		
		"""
		
		if not folder.endswith("/"):
			folder=folder+"/"
			
		if os.path.isdir(folder+fn):
			return False
		else:
			try:
				os.mkdir(folder+fn)
				self.changePermissions(folder+fn,uid,gid,mode)
				return True
			except:
				log.info("Unable to create folder %s" %(folder+fn))
				return False
			
#Define setup

#Check if setup.py is used to build RTD, then don't overwrite install command
if os.environ.get('READTHEDOCS', None) == 'True':
	
	print "Installing on RTD, will not overwrite install command."
	
	setup(name='pyfrp',
		version=version,
		description='PyFRAP: A Python based FRAP analysis tool box',
		url='https://github.com/alexblaessle/PyFRAP',
		author='Alexander Blaessle',
		author_email='alexander.blaessle@tuebingen.mpg.de',
		license='GNU GPL Version 3',
		packages=['pyfrp','pyfrp.modules','pyfrp.subclasses','pyfrp.gui'],
		package_dir={'pyfrp': 'pyfrp',
				'pyfrp.modules': 'pyfrp/modules',
				'pyfrp.gui' : 'pyfrp/gui'
				},
		#package_data = {'pyfrp':['meshfiles','configurations']},
		include_package_data=True,
		classifiers= [
			'Development Status :: 4 - Beta',
			'Operating System :: OS Independent',
			'Programming Language :: Python :: 2.7',
			'Topic :: Scientific/Engineering :: Bio-Informatics',
			'Intended Audience :: Science/Research',
			'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',\
			'Programming Language :: Python :: 2.7',
			],
		install_requires=['pyopenssl','ndg-httpsclient','pyasn1','ez_setup','numpy','scipy','matplotlib','scikit-image','FiPy','colorama','numpy-stl','solidpython','wget','python-bioformats'],	
		platforms=['ALL'],
		keywords=["FRAP", "fluorescence",'recovery','after','photobleaching','reaction-diffusion','fitting'
			],
		zip_safe=False
		)


else:

	setup(name='pyfrp',
		version=version,
		description='PyFRAP: A Python based FRAP analysis tool box',
		url='https://github.com/alexblaessle/PyFRAP',
		author='Alexander Blaessle',
		author_email='alexander.blaessle@tuebingen.mpg.de',
		license='GNU GPL Version 3',
		packages=['pyfrp','pyfrp.modules','pyfrp.subclasses','pyfrp.gui'],
		package_dir={'pyfrp': 'pyfrp',
				'pyfrp.modules': 'pyfrp/modules',
				'pyfrp.gui' : 'pyfrp/gui'
				},
		#package_data = {'pyfrp':['meshfiles','configurations']},
		include_package_data=True,
		classifiers= [
			'Development Status :: 4 - Beta',
			'Operating System :: OS Independent',
			'Programming Language :: Python :: 2.7',
			'Topic :: Scientific/Engineering :: Bio-Informatics',
			'Intended Audience :: Science/Research',
			'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',\
			'Programming Language :: Python :: 2.7',
			],
		install_requires=['pyopenssl','ndg-httpsclient','pyasn1','ez_setup','numpy','scipy','matplotlib','scikit-image','FiPy','colorama','numpy-stl','solidpython','wget','python-bioformats'],	
		platforms=['ALL'],
		keywords=["FRAP", "fluorescence",'recovery','after','photobleaching','reaction-diffusion','fitting'
			],
		zip_safe=False,
		cmdclass={'install': OverrideInstall} #Need this here to overwrite our install
		)



#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#PyQT Dialogs for mesh class
#(1) 

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#QT
from PyQt4 import QtGui, QtCore

#PyFRAP GUI classes
import pyfrp_gui_basics
import pyfrp_gui_builder

#PyFRAP modules
from pyfrp.modules.pyfrp_term_module import *
from pyfrp.modules import pyfrp_img_module
from pyfrp.modules import pyfrp_misc_module

#Numpy/Scipy
import numpy as np

#Misc 
import os


#===================================================================================================================================
#Dialog for editing mesh settings
#===================================================================================================================================

class meshSettingsDialog(pyfrp_gui_basics.basicSettingsDialog):
	
	def __init__(self,mesh,parent):
		
		super(meshSettingsDialog,self).__init__(parent)
		
		self.mesh = mesh
		self.parent=parent
		self.nCharDisplayed=50
		
		#Labels
		self.lblVolSizePx = QtGui.QLabel("Element Size (px):", self)
		self.lblFromFile = QtGui.QLabel("From File?:", self)
		self.lblFnMesh = QtGui.QLabel("Filename mesh:", self)
		
		self.lblFnMeshVal = QtGui.QLabel("", self)
		self.updateFnMeshLbl()
		
		#LineEdits
		self.qleVolSizePx = QtGui.QLineEdit(str(self.mesh.volSizePx))
		
		self.qleVolSizePx.setValidator(self.doubleValid)
		
		self.qleVolSizePx.editingFinished.connect(self.setVolSizePx)
		
		#CheckBox
		self.cbFromFile = QtGui.QCheckBox('', self)
		self.updateCBs()
		self.connect(self.cbFromFile, QtCore.SIGNAL('stateChanged(int)'), self.checkFromFile)
		
		#Buttons
		self.btnFnMesh=QtGui.QPushButton('Change')
		
		self.btnFnMesh.connect(self.btnFnMesh, QtCore.SIGNAL('clicked()'), self.setFnMesh)
		
		#Layout
		self.fnMeshGrid = QtGui.QGridLayout()
		self.fnMeshGrid.addWidget(self.lblFnMeshVal,1,1)
		self.fnMeshGrid.addWidget(self.btnFnMesh,1,2)
		self.fnMeshGrid.setColumnMinimumWidth(1,150)
		
		self.grid.addWidget(self.lblVolSizePx,1,1)
		self.grid.addWidget(self.lblFromFile,2,1)
		self.grid.addWidget(self.lblFnMesh,3,1)
		
		self.grid.addWidget(self.qleVolSizePx,1,2)
		self.grid.addWidget(self.cbFromFile,2,2)
		self.grid.addLayout(self.fnMeshGrid,3,2)
		
		self.setWindowTitle("Mesh Settings")
		
		self.show()
		
	def setVolSizePx(self):
		self.mesh.setVolSizePx(float(str(self.qleVolSizePx.text())))
		
	def setFnMesh(self):
		
		fn = str(QtGui.QFileDialog.getExistingFile(self, "Select Mesh File",  self.parent.lastopen,"*.msh",))
		if fn=='':
			return
		
		self.mesh.setFnMesh(fn)
		
		self.parent.lastopen=folder
		self.updateFnMeshLbl()
		
	def updateFnMeshLbl(self):
		self.lblFnMeshVal.setText("..."+self.mesh.fnMesh[-self.nCharDisplayed:])	
		
	def checkFromFile(self,val):
		self.mesh.setFromFile(bool(2*val))

	def updateCBs(self):
		self.cbFromFile.setCheckState(2*int(self.mesh.fromFile))
	
#===================================================================================================================================
#Dialogs for mesh generation progress
#===================================================================================================================================

class genMeshProgressDialog(pyfrp_gui_basics.waitDialog):
	
	def __init__(self,parent):
		super(genMeshProgressDialog,self).__init__(parent)
		
		#Labels
		self.lblName.setText("Mesh generation in progress...")
		
		#Window title
		self.setWindowTitle('Mesh generation')
		    
		self.show()	

class genMeshThread(pyfrp_gui_basics.pyfrpThread):
	
	def __init__(self, embryo=None, parent=None):
		
		super(genMeshThread,self).__init__(parent)
		self.obj=embryo
		self.embryo=embryo
				
	def runTask(self,debug=False):
		self.embryo.simulation.mesh.genMesh()
		
#===================================================================================================================================
#Dialogs for mesh refine progress
#===================================================================================================================================

class refineMeshProgressDialog(pyfrp_gui_basics.waitDialog):
	
	def __init__(self,parent):
		super(refineMeshProgressDialog,self).__init__(parent)
		
		#Labels
		self.lblName.setText("Mesh refinement in progress...")
		
		#Window title
		self.setWindowTitle('Mesh refinement')
		    
		self.show()	

class refineMeshThread(pyfrp_gui_basics.pyfrpThread):
	
	def __init__(self, embryo=None, parent=None):
		
		super(refineMeshThread,self).__init__(parent)
		self.obj=embryo
		self.embryo=embryo
				
	def runTask(self,debug=False):
		self.embryo.simulation.mesh.refine()


		
#===================================================================================================================================
#Dialogs for forcing mesh density
#===================================================================================================================================

class basicForceMeshSettingsDialog(pyfrp_gui_basics.basicSettingsDialog):
	
	def __init__(self,mesh,parent):
		
		super(basicForceMeshSettingsDialog,self).__init__(parent)
	
		#Pass mesh
		self.mesh = mesh
		self.parent=parent
		self.embryo=self.mesh.simulation.embryo
		
		#Set default variables
		self.roiUsed=None
		self.debug=False
		self.findIdxs=True
		
		#Labels
		self.lblROI = QtGui.QLabel("ROI used:", self)
		self.lblDebug = QtGui.QLabel("Print debugging output:", self)
		self.lblFindIdxs = QtGui.QLabel("Update Indices?:", self)
		
		#LineEdits
		self.doubleValid=QtGui.QDoubleValidator()
		self.intValid=QtGui.QIntValidator()
		
		#Combobox
		self.comboROI = QtGui.QComboBox(self)
		self.updateROICombo()
		
		self.comboROI.activated[str].connect(self.setROI)   
		
		#CheckBox
		self.cbDebug = QtGui.QCheckBox('', self)
		self.cbFindIdxs = QtGui.QCheckBox('', self)
		
		self.connect(self.cbDebug, QtCore.SIGNAL('stateChanged(int)'), self.checkDebug)
		self.connect(self.cbFindIdxs, QtCore.SIGNAL('stateChanged(int)'), self.checkFindIdxs)
		
	
		#Layout	
		self.grid.addWidget(self.lblROI,1,1)
		self.grid.addWidget(self.lblFindIdxs,4,1)
		self.grid.addWidget(self.lblDebug,6,1)
		
		self.grid.addWidget(self.comboROI,1,2)
		self.grid.addWidget(self.cbFindIdxs,4,2)
		self.grid.addWidget(self.cbDebug,6,2)
		
		self.setWindowTitle("Basic Force Mesh Settings")
		
		self.show()
		
	def updateROICombo(self):
		for r in self.embryo.ROIs:
			self.comboROI.addItem(r.name)
		
		self.setROI()
		
	def setROI(self):
		self.roiUsed=self.mesh.simulation.embryo.ROIs[int(self.comboROI.currentIndex())]
	
	def checkDebug(self,val):
		self.debug=bool(2*val)
	
	def checkFindIdxs(self,val):
		self.findIdxs=bool(2*val)
		
	def donePressed(self):
		self.done(1)

class forceMeshSettingsDialog(basicForceMeshSettingsDialog):
	
	def __init__(self,mesh,parent):
		
		super(forceMeshSettingsDialog,self).__init__(mesh,parent)
	
		#Set default variables
		self.method='refine'
		self.stepPercentage=0.1
		self.maxCells=100000
		
		#Labels
		self.lblDensity = QtGui.QLabel("Desired Mesh Density:", self)
		self.lblStepPercentage = QtGui.QLabel("Step Size (%):", self)
		self.lblMethod = QtGui.QLabel("Method:", self)
		self.lblMaxCells = QtGui.QLabel("Maximum number of cells:", self)
		
		#LineEdits
		self.qleDensity = QtGui.QLineEdit("0")
		self.qleMaxCells = QtGui.QLineEdit(str(self.maxCells))
		self.qleStepPercentage = QtGui.QLineEdit(str(self.stepPercentage))
	
		self.qleDensity.setValidator(self.doubleValid)
		self.qleMaxCells.setValidator(self.intValid)
		self.qleStepPercentage.setValidator(self.doubleValid)
		
		self.qleDensity.editingFinished.connect(self.setDensity)
		self.qleMaxCells.editingFinished.connect(self.setMaxCells)
		self.qleStepPercentage.editingFinished.connect(self.setStepPercentage)
		
		#Combobox
		self.comboMethod = QtGui.QComboBox(self)
		self.addItem("refine")
		self.addWidget("method")
		
		self.comboMethod.activated[str].connect(self.setMethod)   
		
		#Layout	
		self.grid.addWidget(self.lblDensity,2,1)
		self.grid.addWidget(self.lblMethod,3,1)
		self.grid.addWidget(self.lblMaxCells,5,1)
		
		self.grid.addWidget(self.qleDensity,2,2)
		self.grid.addWidget(self.comboMethod,3,2)
		self.grid.addWidget(self.qleMaxCells,5,2)
	
		self.setWindowTitle("Force Global Mesh Density Settings")
		
		self.show()
	
	def updateROICombo(self):
		for r in self.embryo.ROIs:
			self.comboROI.addItem(r.name)
		
		self.setROI()
		self.qleDensity.setText(str(self.roiUsed.getMeshDensity()))
	
	def setDensity(self):
		self.density=float(str(self.qleDensity.text()))
	
	def setMaxCells(self):
		self.maxCells=int(str(self.qleMaxCells.text()))
		
	def setMethod(self,text):
		self.method=str(text)
	
	def setStepPercentage(self):
		self.stepPercentage=float(str(qleStepPercentage.text()))
	
	def getVals(self):
		return self.roiUsed,self.density,self.stepPercentage,self.debug,self.findIdxs,self.method,self.maxCells
	
	def donePressed(self):
		self.getVals()
		self.done(1)

#===================================================================================================================================
#Dialogs for refining single ROI by box mesh
#===================================================================================================================================

class refineROIMeshSettingsDialog(basicForceMeshSettingsDialog):
	
	def __init__(self,mesh,parent):
		
		super(refineROIMeshSettingsDialog,self).__init__(mesh,parent)
	
		#Set default variables
		self.factor=3.
		self.addZ=15
		
		#Labels
		self.lblFactor = QtGui.QLabel("Factor :", self)
		self.lblAddZ = QtGui.QLabel("Extend Box Mesh by z(px):", self)
		
		#LineEdits
		self.qleFactor = QtGui.QLineEdit(str(self.factor))
		self.qleAddZ = QtGui.QLineEdit(str(self.addZ))
	
		self.qleAddZ.setValidator(self.doubleValid)
		self.qleFactor.setValidator(self.doubleValid)
		
		self.qleFactor.editingFinished.connect(self.setFactor)
		self.qleAddZ.editingFinished.connect(self.setAddZ)
		
		#Layout
		self.grid.addWidget(self.lblFactor,2,1)
		self.grid.addWidget(self.lblAddZ,3,1)
		
		self.grid.addWidget(self.qleFactor,2,2)
		self.grid.addWidget(self.qleAddZ,3,2)
	
		self.setWindowTitle("Refine ROI Mesh Density Settings")
		
		self.show()
		
	def setAddZ(self):
		self.addZ=float(str(self.qleAddZ.text()))
		
	def setFactor(self):
		self.factor=float(str(self.qleFactor.text()))
	
	def getVals(self):
		return self.roiUsed,self.factor,self.addZ,self.debug,self.findIdxs
	
	def donePressed(self):
		if self.roiUsed==None:
			printWarning("No ROI selected. Going to do nothing.")
		else:
			self.roiUsed.refineInMeshByField(factor=self.factor,addZ=self.addZ,findIdxs=self.findIdxs,debug=self.debug)
		
		self.done(1)


#===================================================================================================================================
#Dialogs for BL mesh
#===================================================================================================================================

class boundaryLayerAroundROISettingsDialog(pyfrp_gui_basics.basicSettingsDialog):
	
	def __init__(self,embryo,parent):
		
		super(boundaryLayerAroundROISettingsDialog,self).__init__(parent)
	
		# Set default variables
		self.embryo=embryo
		self.initDefaults()
		
		# Generate all widgets
		self.lblSegments,self.qleSegments = pyfrp_gui_builder.genSettingQLE(self,"Segments",self.segments,callback=self.setSegments,validator=self.intValid)
		self.lblIter,self.qleIter = pyfrp_gui_builder.genSettingQLE(self,"Iterations",self.iterations,callback=self.setIter,validator=self.intValid)
		self.lblTriangIter,self.qleTriangIter = pyfrp_gui_builder.genSettingQLE(self,"triangIterations",self.triangIterations,callback=self.setTriangIter,validator=self.intValid)
		self.lblVolSizePx,self.qleVolSizePx = pyfrp_gui_builder.genSettingQLE(self,"volSizePx",self.volSizePx,callback=self.setVolSizePx,validator=self.doubleValid)
		self.lblVolSizeLayer,self.qleVolSizeLayer = pyfrp_gui_builder.genSettingQLE(self,"volSizeLayer",self.volSizeLayer,callback=self.setVolSizeLayer,validator=self.doubleValid)
		self.lblThickness,self.qleThickness = pyfrp_gui_builder.genSettingQLE(self,"thickness",self.thickness,callback=self.setThickness,validator=self.doubleValid)
		self.lblAngleThresh,self.qleAngleThresh = pyfrp_gui_builder.genSettingQLE(self,"angleThresh",self.angleThresh,callback=self.setAngleThresh,validator=self.doubleValid)
		self.lblFaces,self.qleFaces = pyfrp_gui_builder.genSettingQLE(self,"Faces",self.faces,callback=self.setFaces,validator=None)
		self.lblFnOut,self.btnFnOut = pyfrp_gui_builder.genSettingBtn(self,"fnOut","Change",callback=self.setFnOut)
		
		self.lblOnlyAbs,self.cbOnlyAbs = pyfrp_gui_builder.genSettingCB(self,"onlyAbs",self.onlyAbs,callback=self.checkOnlyAbs)
		self.lblSimplify,self.cbSimplify = pyfrp_gui_builder.genSettingCB(self,"simplify",self.simplify,callback=self.checkSimplify)
		self.lblFixSurf,self.cbFixSurf = pyfrp_gui_builder.genSettingCB(self,"fixSurfaces",self.fixSurfaces,callback=self.checkFixSurf)
		self.lblApproxBySpline,self.cbApproxBySpline = pyfrp_gui_builder.genSettingCB(self,"approxBySpline",self.approxBySpline,callback=self.checkApproxBySpline)
		self.lblDebug,self.cbDebug = pyfrp_gui_builder.genSettingCB(self,"debug",self.debug,callback=self.checkDebug)
		self.lblCleanUp,self.cbCleanUp = pyfrp_gui_builder.genSettingCB(self,"cleanUp",self.cleanUp,callback=self.checkCleanUp)
		
		self.lblROI,self.comboROI = pyfrp_gui_builder.genSettingCombo(self,"ROI",pyfrp_misc_module.objAttrToList(self.embryo.ROIs,"name"),callback=self.setROI,idx=self.embryo.getROIIdx(self.roiUsed))
		
		
		# Put them in list 
		lbls1 = [self.lblROI,self.lblFaces,self.lblOnlyAbs,self.lblFnOut]
		qles1 = [self.comboROI,self.qleFaces,self.cbOnlyAbs,self.btnFnOut]
		lbls2 = [self.lblVolSizePx,self.lblVolSizeLayer,self.lblThickness]
		qles2 = [self.qleVolSizePx,self.qleVolSizeLayer,self.qleThickness]
		lbls3 = [self.lblSegments,self.lblIter,self.lblTriangIter]
		qles3 = [self.qleSegments,self.qleIter,self.qleTriangIter]
		lbls4 = [self.lblSimplify,self.lblFixSurf,self.lblApproxBySpline,self.lblAngleThresh]
		qles4 = [self.cbSimplify,self.cbFixSurf,self.cbApproxBySpline,self.qleAngleThresh]
		lbls5 = [self.lblDebug,self.lblCleanUp]
		qles5 = [self.cbDebug,self.cbCleanUp]
		
		lbls=[lbls1,lbls2,lbls3,lbls4,lbls5]
		qles=[qles1,qles2,qles3,qles4,qles5]
		
		# Add to Layout
		nRows=self.grid.rowCount()
		for i in range(len(lbls)):
			for j in range(len(lbls[i])):
				self.grid.addWidget(lbls[i][j],nRows+j+1,(2*i))
				self.grid.addWidget(qles[i][j],nRows+j+1,(2*i+1))
			
		self.setWindowTitle("Boundary Layer Mesh around ROI settings")
		
		self.show()
	
	def initDefaults(self):
		
		"""Sets default parameters that are generally passed on."""
		
		self.segments=48
		self.simplify=True
		self.iterations=3
		self.triangIterations=2
		self.fixSurfaces=True
		self.debug=False
		self.volSizePx=self.embryo.simulation.mesh.getVolSizePx()
		self.thickness=15.
		self.volSizeLayer=self.volSizePx/2.
		self.cleanUp=True
		self.approxBySpline=True
		self.angleThresh=0.95
		self.faces=['x','y']
		self.onlyAbs=True
		self.fnOut=self.embryo.geometry.fnGeo.replace(".geo","_BL.geo")
		
		self.roiUsed=self.embryo.getROIByName("All Bleached Square")
		if self.roiUsed==None:
			self.roiUsed=self.embryo.ROIs[0]
		
	def setSegments(self):
		self.segments=int(str(self.qleSegments.text()))
	
	def setIter(self):
		self.iterations=int(str(self.qleIter.text()))
	
	def setTriangIter(self):
		self.triangIterations=int(str(self.qleTriangIter.text()))
	
	def setVolSizePx(self):
		self.volSizePx=float(str(self.qleVolSizePx.text()))
	
	def setVolSizeLayer(self):
		self.volSizeLayer=float(str(self.qleVolSizeLayer.text()))
	
	def setThickness(self):
		self.thickness=float(str(self.qleThickness.text()))
	
	def setAngleThresh(self):
		self.angleThresh=float(str(self.qleAngleThresh.text()))
	
	def setFaces(self):
		text==str(self.qleFaces.text())
		if text=="all":
			self.faces=text
			return
		
		try:
			self.faces=pyfrp_misc_module.str2list(text,dtype="float")[0]
			return 
		except ValueError:
			try:
				self.faces=pyfrp_misc_module.str2list(text,dtype="str")[0]
				return
			except:
				pass
		
		printError("Wasn't able to set faces for input: " + text)
	
	def checkApproxBySpline(self,val):
		self.approxBySpline=bool(2*val)
	
	def checkDebug(self,val):
		self.debug=bool(2*val)
	
	def checkCleanUp(self,val):
		self.cleanUp=bool(2*val)
	
	def checkSimplify(self,val):
		self.simplify=bool(2*val)
	
	def checkOnlyAbs(self,val):
		self.onlyAbs=bool(2*val)
	
	def checkFixSurf(self,val):
		self.fixSurfaces=bool(2*val)
	
	def setROI(self):
		self.roiUsed=self.embryo.ROIs[int(self.comboROI.currentIndex())]
	
	def setFnOut(self):
		
		fn=QtGui.QFileDialog.getSaveFileName(self, 'Path to boundary layer geo file', self.fnOut,"*.geo",)
		self.fnOut=str(fn)
		
	def getVals(self):
		return self.fnOut,self.roiUsed,self.segments,self.simplify,self.iterations,self.triangIterations,self.fixSurfaces,self.debug,self.volSizePx,self.thickness,self.volSizeLayer,self.cleanUp,self.approxBySpline,self.angleThresh,self.faces,self.onlyAbs
		
	def donePressed(self):
		if self.roiUsed==None:
			printWarning("No ROI selected. Please select ROI first.")
		else:
			self.done(1)	
			
#===================================================================================================================================
#Dialogs for forcing mesh density progress
#===================================================================================================================================

class forceMeshProgressDialog(pyfrp_gui_basics.waitDialog):
	
	def __init__(self,parent):
		super(forceMeshProgressDialog,self).__init__(parent)
		
		#Labels
		self.lblName.setText("Force mesh density in progress...")
		
		#Window title
		self.setWindowTitle('Mesh density enforcing')
		    
		self.show()	

class boundaryLayerProgressDialog(pyfrp_gui_basics.waitDialog):
	
	def __init__(self,parent):
		super(boundaryLayerProgressDialog,self).__init__(parent)
		
		#Labels
		self.lblName.setText("Boundary layer meshing in progress...")
		
		#Window title
		self.setWindowTitle('Boundary layer meshing')
		    
		self.show()	

class forceMeshThread(pyfrp_gui_basics.pyfrpThread):
	
	def __init__(self, embryo=None, parent=None):
		
		super(forceMeshThread,self).__init__(parent)
		self.obj=embryo
		self.embryo=embryo
		
	def runTask(self,roi,debug=False):
		self.embryo.simulation.mesh.forceMinMeshDensityInROI(roi)
		
				
		
			
			
		
		
				
		
		